@@ -1,7 +1,4 @@
-/* dbops.c
- * Database operations.
- * This file is part of the edbrowse project, released under GPL.
- */
+/* dbops.c, database operations. */
 
 #include "eb.h"
 #include "dbapi.h"
@@ -332,18 +329,42 @@ put them back into binary, ready for retsCopy().
 char *sql_mkunld(char delim)
 {
 	char fmt[NUMRETS * 4 + 1];
-	int i;
+	int i, np;
 	char pftype;
+	char *unld;
 
 	for (i = 0; i < rv_numRets; ++i) {
 		pftype = sprintfChar(rv_type[i]);
 		if (!pftype)
 			errorPrint("2sql_mkunld cannot convert datatype %c",
 				   rv_type[i]);
-		sprintf(fmt + 4 * i, "%%0%c%c", pftype, delim);
+		sprintf(fmt + 4 * i, "%%0%c%c", pftype, '\177');
 	}			/* loop over returns */
 
-	return lineFormatStack(fmt, rv_data, 0);
+	unld = lineFormatStack(fmt, rv_data, 0);
+
+// I assume delete was not in any of the strings.
+// Should probably use 0xff instead but that is nonascii and someone
+// might try to turn it into utf8 and idk.
+	for(i= np =0; unld[i]; ++i)
+		if(unld[i] == delim) ++np;
+	if(i + np >= LFBUFSIZE)
+		errorPrint(lfoverflow, LFBUFSIZE);
+	for(i = 0; unld[i]; ++i) {
+		char c = unld[i];
+		if(c == '\177') {
+			unld[i] = delim;
+			if(i && unld[i-1] == '\\')
+				return 0;
+		}
+		if(c == delim) {
+			strmove(unld + i + 1, unld + i);
+			unld[i] = '\\';
+			++i;
+		}
+	}
+
+	return unld;
 }
 
 /* like the above, but we build a comma-separated list with quotes,
@@ -691,8 +712,56 @@ static int wcllen;
 static char wherecol[COLNAMELEN + 2];
 static struct DBTABLE *td;
 
-/* put quotes around a column value */
-static void pushQuoted(char **s, int *slen, const char *value, int colno)
+/*********************************************************************
+See if a line is part of an unfolded row.
+It must begin with a column name followed by a colon.
+Then back up to see if the unfolded row begins with the row starter,
+Return the column number, or -1 if it is an inline row.
+*********************************************************************/
+
+static const char RowStart[] = "@row:\n";
+static const int RowStartLen = 6;
+
+static int isUnfolded(int ln)
+{
+	int j;
+	unsigned l;
+	const pst line = fetchLine(ln, -1);
+	const char *s = (const char *)line;
+	const char *t = s;
+	if(!memcmp(s, RowStart, RowStartLen)) return 0;
+	if(!isalphaByte(*t)) return -1;
+	for(++t; *t; ++t) {
+		if(*t == ':') break;
+		if(!isalnumByte(*s) && *s != '_')
+			return -1;
+	}
+// has to match a column name
+	l = t - s;
+	for(j = 0; j < td->ncols; ++j) {
+		const char *w = td->cols[j];
+		if(l == strlen(w) && !memcmp(w, s, l)) break;
+	}
+	if(j == td->ncols) return -1;
+// looks good, back up and look for top of row
+	s = (const char *)fetchLine(ln - j - 1, -1);
+	return !memcmp(s, RowStart, RowStartLen) ? j + 1 : -1;
+}
+
+// return 1 if the start of a row, 2 if the end of a row
+int unfoldRowCheck(int ln)
+{
+	int j;
+	if(!ln || !cw->sqlMode) return 3;
+	j = isUnfolded(ln);
+	if(j < 0) return 3;
+	if(j == 0) return 1;
+	if(j == td->ncols) return 2;
+	return 0;
+}
+
+// put quotes around a column value
+static void pushQuoted(char **s, int *slen, const char *value, int colno, bool fixpipe)
 {
 	char quotemark = 0;
 	char coltype = td->types[colno];
@@ -703,17 +772,43 @@ static void pushQuoted(char **s, int *slen, const char *value, int colno)
 	}
 
 	if (coltype != 'F' && coltype != 'N') {
-/* Microsoft insists on single quote. */
 		quotemark = '\'';
-		if (strchr(value, quotemark))
+// I hear microsoft sql server requires single quote, idk
+		if(strchr(value, quotemark))
 			quotemark = '"';
 	}
 	if (quotemark)
 		stringAndChar(s, slen, quotemark);
-	stringAndString(s, slen, value);
+// do we have to escape stuff?
+	if((quotemark && strchr(value, quotemark)) ||
+	(fixpipe && strchr(value, '|'))) {
+		const char *w = value;
+		while(*w) {
+			if(*w == '\\' && w[1] == '|' && fixpipe) {
+				++w;
+				continue;
+			}
+			if(*w == quotemark)
+				stringAndChar(s, slen, quotemark);
+			stringAndChar(s, slen, *w++);
+		}
+	} else {
+		stringAndString(s, slen, value);
+	}
 	if (quotemark)
 		stringAndChar(s, slen, quotemark);
-}				/* pushQuoted */
+}
+
+static void push1field(char **s, int *slen, int ln, int colno)
+{
+	unsigned l;
+	char*v = (char*)fetchLine(ln, -1);
+	v = strchr(v, ':') + 1;
+	l = pstLength((pst)v) - 1;
+	v[l] = 0; // I'll put it bacck
+	pushQuoted(s, slen, v, colno, false);
+	v[l] = '\n';
+}
 
 static char *lineFields[MAXTCOLS];
 
@@ -721,8 +816,7 @@ static char *keysQuoted(void)
 {
 	char *u;
 	int ulen;
-	int key1 = td->key1, key2 = td->key2;
-	int key3 = td->key3;
+	int key1 = td->key1, key2 = td->key2, key3 = td->key3;
 	if (!key1)
 		return 0;
 	u = initString(&ulen);
@@ -730,7 +824,7 @@ static char *keysQuoted(void)
 	stringAndString(&u, &ulen, "where ");
 	stringAndString(&u, &ulen, td->cols[key1]);
 	stringAndString(&u, &ulen, " = ");
-	pushQuoted(&u, &ulen, lineFields[key1], key1);
+	pushQuoted(&u, &ulen, lineFields[key1], key1, true);
 	if (!key2)
 		return u;
 
@@ -738,7 +832,7 @@ static char *keysQuoted(void)
 	stringAndString(&u, &ulen, " and ");
 	stringAndString(&u, &ulen, td->cols[key2]);
 	stringAndString(&u, &ulen, " = ");
-	pushQuoted(&u, &ulen, lineFields[key2], key2);
+	pushQuoted(&u, &ulen, lineFields[key2], key2, true);
 	if (!key3)
 		return u;
 
@@ -746,9 +840,41 @@ static char *keysQuoted(void)
 	stringAndString(&u, &ulen, " and ");
 	stringAndString(&u, &ulen, td->cols[key3]);
 	stringAndString(&u, &ulen, " = ");
-	pushQuoted(&u, &ulen, lineFields[key3], key3);
+	pushQuoted(&u, &ulen, lineFields[key3], key3, true);
 	return u;
-}				/* keysQuoted */
+}
+
+static char *keysUnfoldQuoted(int ln0)
+{
+	char *u;
+	int ulen;
+	int key1 = td->key1, key2 = td->key2, key3 = td->key3;
+	if (!key1)
+		return 0;
+	u = initString(&ulen);
+	--key1;
+	stringAndString(&u, &ulen, "where ");
+	stringAndString(&u, &ulen, td->cols[key1]);
+	stringAndString(&u, &ulen, " = ");
+	push1field(&u, &ulen, ln0 + key1 + 1, key1);
+	if (!key2)
+		return u;
+
+	--key2;
+	stringAndString(&u, &ulen, " and ");
+	stringAndString(&u, &ulen, td->cols[key2]);
+	stringAndString(&u, &ulen, " = ");
+	push1field(&u, &ulen, ln0 + key2 + 1, key2);
+	if (!key3)
+		return u;
+
+	--key3;
+	stringAndString(&u, &ulen, " and ");
+	stringAndString(&u, &ulen, td->cols[key3]);
+	stringAndString(&u, &ulen, " = ");
+	push1field(&u, &ulen, ln0 + key3 + 1, key3);
+	return u;
+}
 
 static void buildSelectClause(void)
 {
@@ -762,7 +888,7 @@ static void buildSelectClause(void)
 	}
 	stringAndString(&scl, &scllen, " from ");
 	stringAndString(&scl, &scllen, td->name);
-}				/* buildSelectClause */
+}
 
 static char date2buf[24];
 static bool dateBetween(const char *s)
@@ -778,7 +904,7 @@ static bool dateBetween(const char *s)
 		return false;
 	*e = 0;
 	return stringIsDate(date2buf) | stringIsDate(e + 1);
-}				/* dateBetween */
+}
 
 static bool buildWhereClause(void)
 {
@@ -865,21 +991,22 @@ setcol_n:
 		stringAndString(&wcl, &wcllen, "\" and \"");
 		stringAndString(&wcl, &wcllen, date2buf + strlen(date2buf) + 1);
 		stringAndChar(&wcl, &wcllen, '"');
-	} else if (w[strlen(w) - 1] == '*') {
-		stringAndString(&wcl, &wcllen, lineFormat(" matches %S", w));
+	} else if (w[strlen(w) - 1] == '%') {
+		stringAndString(&wcl, &wcllen, lineFormat(" like %S", w));
 	} else {
 		stringAndString(&wcl, &wcllen, " = ");
-		pushQuoted(&wcl, &wcllen, w, colno - 1);
+		pushQuoted(&wcl, &wcllen, w, colno - 1, false);
 	}
 
 	return true;
-}				/* buildWhereClause */
+}
 
 static bool setTable(void)
 {
 	static const short exclist[] = { EXCNOTABLE, EXCNOCOLUMN, 0 };
 	int cid, nc, i, part1, part2, part3, part4;
 	const char *s = cf->fileName;
+	++s; // don't need the leading ]
 	const char *t = strchr(s, ']');
 	if ((unsigned)(t - s) >= sizeof(myTab))
 		errorPrint("2table name too long, limit %d characters",
@@ -930,9 +1057,7 @@ static bool setTable(void)
 		}
 		nc = rv_numRets;
 		if (nc > MAXTCOLS) {
-			printf
-			    ("warning, only the first %d columns will be selected\n",
-			     MAXTCOLS);
+			i_printf(MSG_FirstColumns, MAXTCOLS);
 			nc = MAXTCOLS;
 		}
 		td->types = cloneString(rv_type);
@@ -961,7 +1086,7 @@ static bool setTable(void)
 
 	cw->table = td;
 	return true;
-}				/* setTable */
+}
 
 void showColumns(void)
 {
@@ -1023,7 +1148,7 @@ void showColumns(void)
 		}		/* switch */
 		printf("%s\n", desc);
 	}
-}				/* showColumns */
+}
 
 void showForeign(void)
 {
@@ -1031,7 +1156,7 @@ void showForeign(void)
 		return;
 	i_printf(MSG_Fkeys, td->name);
 	fetchForeign(td->name);
-}				/* showForeign */
+}
 
 /* Select rows of data and put them into the text buffer */
 static bool rowsIntoBuffer(int cid, const char *types, char **bufptr, int *lcnt)
@@ -1045,18 +1170,16 @@ static bool rowsIntoBuffer(int cid, const char *types, char **bufptr, int *lcnt)
 	rbuf = initString(&rbuflen);
 
 	while (sql_fetchNext(cid, 0)) {
-		unld = sql_mkunld('\177');
-		if (strchr(unld, '|')) {
-			setError(MSG_DBPipes);
+		unld = sql_mkunld('|');
+		if(!unld) {
+			setError(MSG_DBBackslash);
 			goto abort;
 		}
 		if (strchr(unld, '\n')) {
 			setError(MSG_DBNewline);
 			goto abort;
 		}
-		for (s = unld; *s; ++s)
-			if (*s == '\177')
-				*s = '|';
+		s = unld + strlen(unld);
 		s[-1] = '\n';	/* overwrite the last pipe */
 
 /* look for blob column */
@@ -1090,7 +1213,7 @@ abort:
 	sql_closeFree(cid);
 	*bufptr = rbuf;
 	return rc;
-}				/* rowsIntoBuffer */
+}
 
 bool sqlReadRows(const char *filename, char **bufptr)
 {
@@ -1102,7 +1225,7 @@ bool sqlReadRows(const char *filename, char **bufptr)
 	if (!setTable())
 		return false;
 
-	myWhere = strchr(filename, ']') + 1;
+	myWhere = strchr(filename + 1, ']') + 1;
 	if (!*myWhere)
 		return true;
 
@@ -1117,7 +1240,7 @@ bool sqlReadRows(const char *filename, char **bufptr)
 		return false;
 
 	return rowsIntoBuffer(cid, td->types, bufptr, &lcnt);
-}				/* sqlReadRows */
+}
 
 /* Split a line at pipe boundaries, and make sure the field count is correct */
 static bool intoFields(char *line)
@@ -1128,7 +1251,12 @@ static bool intoFields(char *line)
 
 	while (1) {
 		lineFields[j] = s;
+step:
 		s = strpbrk(s, "|\n");
+		if(*s == '|' && s > line && s[-1] == '\\') {
+			++s;
+			goto step;
+		}
 		c = *s;
 		*s++ = 0;
 		++j;
@@ -1144,7 +1272,120 @@ static bool intoFields(char *line)
 		return true;
 	setError(MSG_DBLostField);
 	return false;
-}				/* intoFields */
+}
+
+void sql_unfold(int start, int end, char action)
+{
+	int ln, ln2, newdol = 0;
+	struct lineMap *newmap;
+	int j, nc = td->ncols, len2;
+	const char *s, *s0;
+	char *v, *w;
+	bool changes = false;
+
+// how many lines in the buffer after the operation?
+	for(ln = 1; ln <= cw->dol; ++ln) {
+		s = (const char *)fetchLine(ln, -1);
+		if(!memcmp(s, RowStart, RowStartLen)) { // row is unfolded
+			if((action == '-' || action == 0) &&
+			((start <= ln && end >= ln) ||
+			(start <= ln + nc && end >= ln + nc) ||
+			(start > ln && end < ln + nc)))
+				++newdol, changes = true;
+			else
+				newdol += nc + 1;
+			ln += nc;
+			continue;
+		}
+		if((action == '+' || action == 0) &&
+		start <= ln && end >= ln)
+			newdol += nc + 1, changes = true;
+		else
+			++newdol;
+	}
+	debugPrint(3, "newdol %d", newdol);
+
+	if(!changes)
+		return;
+
+	newmap = allocZeroMem(LMSIZE * (newdol + 2));
+	ln2 = 1;
+
+// All this text manipulation would be so much easier in perl.
+
+	for(ln = 1; ln <= cw->dol; ++ln) {
+		s = (const char *)fetchLine(ln, -1);
+		if(!memcmp(s, RowStart, RowStartLen)) { // row is unfolded
+			if((action == '-' || action == 0) &&
+			((start <= ln && end >= ln) ||
+			(start <= ln + nc && end >= ln + nc) ||
+			(start > ln && end < ln + nc))) {
+				free((char*)s);
+// how many pipes do we need to escape?
+				len2 = 0;
+				for(j = 1; j <= nc; ++j) {
+					s = (const char *)fetchLine(ln + j, -1);
+					s = strchr(s, ':') + 1;
+					for(; *s != '\n'; ++s, ++len2)
+						if(*s == '|')
+							++len2;
+					++len2;
+				}
+				v = w = allocMem(len2);
+				for(j = 1; j <= nc; ++j) {
+					s0 = s = (const char *)fetchLine(ln + j, -1);
+					s = strchr(s, ':') + 1;
+					for(; *s != '\n'; ++s) {
+						if(*s == '|') *w++ = '\\';
+						*w++ = *s;
+					}
+					*w++ = '|';
+					free((char*)s0);
+				}
+				w[-1] = '\n';
+				cw->dot = ln2;
+				newmap[ln2++].text = (pst)v;
+			} else {
+// no change, just copy
+				for(j = 0; j <= nc; ++j)
+					newmap[ln2 + j].text = cw->map[ln + j].text;
+				ln2 += nc + 1;
+			}
+			ln += nc;
+			continue;
+		}
+		if((action == '+' || action == 0) &&
+		start <= ln && end >= ln) {
+			cw->dot = ln2;
+			newmap[ln2++].text = clonePstring((pst)RowStart);
+// I'm going to muck with the line, cause we're going to free it anyways.
+			intoFields((char*)s);
+			for(j = 0; j < nc; ++j) {
+				s0 = lineFields[j];
+				len2 = strlen(s0) + strlen(td->cols[j]) + 2;
+				v = w = allocMem(len2);
+				strcpy(v, td->cols[j]);
+				w = v + strlen(v);
+				*w++ = ':';
+				for(; *s0; ++s0) {
+					if(*s0 == '\\' && s0[1] == '|') continue;
+				*w++ = *s0;
+				}
+				*w = '\n';
+				newmap[ln2 + j].text = (pst)v;
+			}
+			ln2 += nc;
+			free((char*)s);
+		} else {
+// no change, just copy
+			newmap[ln2++].text = cw->map[ln].text;
+		}
+	}
+
+	free(cw->map);
+	cw->map = newmap;
+	cw->dol = newdol;
+}
 
 static bool rowCountCheck(int action, int cnt1)
 {
@@ -1155,7 +1396,7 @@ static bool rowCountCheck(int action, int cnt1)
 
 	setError(MSG_DBDeleteCount + action, cnt1, cnt2);
 	return false;
-}				/* rowCountCheck */
+}
 
 static int keyCountCheck(void)
 {
@@ -1171,7 +1412,7 @@ static int keyCountCheck(void)
 		return 3;
 	setError(MSG_DBManyKeyCol);
 	return 0;
-}				/* keyCountCheck */
+}
 
 /* Typical error conditions for insert update delete */
 static const short insupdExceptions[] = {
@@ -1220,7 +1461,7 @@ static bool insupdError(int action, int rcnt)
 	}
 
 	return rowCountCheck(action, rcnt);
-}				/* insupdError */
+}
 
 bool sqlDelRows(int start, int end)
 {
@@ -1245,6 +1486,10 @@ bool sqlDelRows(int start, int end)
  * I have to write the one-line-at-a-time code anyways,
  * I'll just use that for now. */
 	while (ndel--) {
+		if(unfoldRowCheck(ln) != 3) {
+			setError(MSG_DelUnfold);
+			return false;
+		}
 		char *wherekeys;
 		char *line = (char *)fetchLine(ln, 0);
 		intoFields(line);
@@ -1259,20 +1504,30 @@ bool sqlDelRows(int start, int end)
 	}
 
 	return true;
-}				/* sqlDelRows */
+}
 
-bool sqlUpdateRow(pst source, int slen, pst dest, int dlen)
+bool sqlUpdateRow(int ln, pst source, int slen, pst dest, int dlen)
 {
+	int ui, ln0;
 	char *d2;		/* clone of dest */
 	char *wherekeys;
 	char *s, *t;
-	int j, l1, l2, nkeys, key1, key2;
+	int j, l1, l2, nkeys, key1, key2, key3;
 	char *u1;		/* column=value of the update statement */
 	int u1len;
 
+// find the unfold index
+	ui = isUnfolded(ln);
+	ln0 = (ui >= 0 ? ln - ui : 0);
+
 /* compare all the way out to newline, so we know both strings end at the same time */
 	if (slen == dlen && !memcmp(source, dest, slen + 1))
-		return true;
+		return true; // no change
+
+	if(dlen && dest[dlen-1] == '\\') {
+		setError(MSG_EndBackslash);
+		return false;
+	}
 
 	if (!setTable())
 		return false;
@@ -1282,6 +1537,44 @@ bool sqlUpdateRow(pst source, int slen, pst dest, int dlen)
 		return false;
 	key1 = td->key1 - 1;
 	key2 = td->key2 - 1;
+	key3 = td->key3 - 1;
+	u1 = initString(&u1len);
+
+	if(ln0) {
+		int clen;
+// updating a field in an unfolded row
+		debugPrint(3, "foldupdate base %d\n", ln0);
+		j = ui - 1;
+		if (ln == ln0 || j == key1 || j == key2 || j == key3) {
+			setError(MSG_DBChangeKey);
+			return false;
+		}
+		clen = strlen(td->cols[j]);
+		if(dlen <= clen || memcmp(dest, td->cols[j], clen) || dest[clen] != ':') {
+			setError(MSG_UpdateColname);
+			return false;
+		}
+		if (td->types[j] == 'B') {
+			setError(MSG_DBChangeBlob);
+			return false;
+		}
+		if (td->types[j] == 'T') {
+			setError(MSG_DBChangeText);
+			return false;
+		}
+		stringAndString(&u1, &u1len, td->cols[j]);
+		stringAndString(&u1, &u1len, " = ");
+		dest += clen + 1;
+		clen = pstLength(dest) - 1;
+		dest[clen] = 0; // I'll put it back
+		pushQuoted(&u1, &u1len, (char*)dest, j, false);
+		dest[clen] = '\n';
+		wherekeys = keysUnfoldQuoted(ln0);
+		sql_exclist(insupdExceptions);
+		sql_exec("update %s set %s %s", td->name, u1, wherekeys);
+		nzFree(wherekeys), nzFree(u1);
+		return insupdError(2, 1);
+	}
 
 	d2 = (char *)clonePstring(dest);
 	if (!intoFields(d2)) {
@@ -1290,15 +1583,19 @@ bool sqlUpdateRow(pst source, int slen, pst dest, int dlen)
 	}
 
 	j = 0;
-	u1 = initString(&u1len);
 	s = (char *)source;
-
 	while (1) {
-		t = strpbrk(s, "|\n");
+		t = s;
+step:
+		t = strpbrk(t, "|\n");
+		if(*t == '|' && t > (char*)source && t[-1] == '\\') {
+			++t;
+			goto step;
+		}
 		l1 = t - s;
 		l2 = strlen(lineFields[j]);
 		if (l1 != l2 || memcmp(s, lineFields[j], l1)) {
-			if (j == key1 || j == key2) {
+			if (j == key1 || j == key2 || j == key3) {
 				setError(MSG_DBChangeKey);
 				goto abort;
 			}
@@ -1314,7 +1611,7 @@ bool sqlUpdateRow(pst source, int slen, pst dest, int dlen)
 				stringAndString(&u1, &u1len, ", ");
 			stringAndString(&u1, &u1len, td->cols[j]);
 			stringAndString(&u1, &u1len, " = ");
-			pushQuoted(&u1, &u1len, lineFields[j], j);
+			pushQuoted(&u1, &u1len, lineFields[j], j, true);
 		}
 
 		if (*t == '\n')
@@ -1338,7 +1635,7 @@ abort:
 	nzFree(d2);
 	nzFree(u1);
 	return false;
-}				/* sqlUpdateRow */
+}
 
 bool sqlAddRows(int ln)
 {
@@ -1347,6 +1644,7 @@ bool sqlAddRows(int ln)
 	char *unld, *s;
 	int u1len, u2len, u3len;
 	int j, l;
+	int p, np; // number of pipes
 	double dv;
 	char inp[256];
 	bool rc;
@@ -1354,6 +1652,13 @@ bool sqlAddRows(int ln)
 	if (!setTable())
 		return false;
 
+// Don't add a row in the middle of an unfolded row.
+	if(ln) {
+		j = isUnfolded(ln);
+		if(j >= 0)
+			ln += td->ncols - j;
+	}
+
 	while (1) {
 		u1 = initString(&u1len);
 		u2 = initString(&u2len);
@@ -1372,6 +1677,10 @@ reenter:
 			l = strlen(inp);
 			if (l && inp[l - 1] == '\n')
 				inp[--l] = 0;
+			if(l && inp[l-1] == '\\') {
+				i_puts(MSG_EndBackslash);
+				goto reenter;
+			}
 			if (stringEqual(inp, ".")) {
 				nzFree(u1);
 				nzFree(u2);
@@ -1387,12 +1696,7 @@ reenter:
 				goto goodfield;
 			}
 
-/* verify the integrity of the entered field */
-			if (strchr(inp, '|')) {
-				puts("please, no pipes in the data");
-				goto reenter;
-			}
-
+// verify the integrity of the entered field
 			switch (td->types[j]) {
 			case 'N':
 				s = inp;
@@ -1431,7 +1735,7 @@ reenter:
 
 goodfield:
 
-/* turn 0 into next serial number */
+// turn 0 into next serial number
 			if (j == td->key1 - 1 && td->types[j] == 'N' &&
 			    stringEqual(inp, "0")) {
 				int nextkey =
@@ -1444,13 +1748,30 @@ goodfield:
 				sprintf(inp, "%d", nextkey + 1);
 			}
 
+// count pipes in input
+			for(p = np = 0; inp[p]; ++p)
+				if(inp[p] == '|') ++np;
+			if((unsigned)(np + l) >= sizeof(inp)) {
+				puts("string becomes too long when pipes are escaped");
+				goto reenter;
+			}
+			if(np) {
+// this is inefficient but who cares.
+				for(p = 0; inp[p]; ++p) {
+					if(inp[p] == '|') {
+						strmove(inp + p + 1, inp + p);
+						inp[p++] = '\\';
+					}
+				}
+			}
+
 			if (*u1)
 				stringAndChar(&u1, &u1len, ',');
 			stringAndString(&u1, &u1len, td->cols[j]);
 
 			if (*u2)
 				stringAndChar(&u2, &u2len, ',');
-			pushQuoted(&u2, &u2len, inp, j);
+			pushQuoted(&u2, &u2len, inp, j, true);
 
 			stringAndString(&u3, &u3len, inp);
 			stringAndChar(&u3, &u3len, '|');
@@ -1474,6 +1795,7 @@ goodfield:
 		sql_select("%s where rowid = %d", scl, rowid, 0);
 		nzFree(scl);
 		unld = sql_mkunld('|');
+// check for unld == null
 		l = strlen(unld);
 		unld[l - 1] = '\n';	/* overwrite the last pipe */
 #else
@@ -1491,7 +1813,7 @@ goodfield:
 
 /* This pointis not reached; make the compilerhappy */
 	return true;
-}				/* sqlAddRows */
+}
 
 /*********************************************************************
 run the analog of /bin/comm on two open cursors,
@@ -1503,8 +1825,9 @@ The cursors should be sorted by this key.
 static void cursor_comm(const char *stmt1, const char *stmt2,	/* the two select statements */
 			const char *orderby,	/* which fetched column is the unique key */
 			int (*f)(char, char*, char*, long),	/* call this function for differences */
+// sql_mkunld() delimiter, or call mkinsupd if delim = 0
 			char delim)
-{				/* sql_mkunld() delimiter, or call mkinsupd if delim = 0 */
+{
 	short cid1, cid2;	/* the cursor ID numbers */
 	char *line1, *line2, *s;	/* the two fetched rows */
 	void *blob1, *blob2;	/* one blob per table */
@@ -1690,7 +2013,7 @@ static int syncup_comm_fn(char action, char *line1, char *line2, long key)
 }				/* syncup_comm_fn */
 
 /* make table1 look like table2 */
-void syncup_table(const char *table1, const char *table2,	/* the two tables */
+void syncup_table(const char *table1, const char *table2,	// the two tables
 		  const char *keycol,	/* the key column */
 		  const char *otherclause)
 {
@@ -1726,7 +2049,7 @@ void syncup_table(const char *table1, const char *table2,	/* the two tables */
 	}
 
 	cursor_comm(stmt1, stmt2, keycol,  syncup_comm_fn, 0);
-}				/* syncup_table */
+}
 
 int goSelect(int *startLine, char **rbuf)
 {
