@@ -1,7 +1,4 @@
-/* eb.h
- * Copyright (c) Karl Dahlke, 2008
- * This file is part of the edbrowse project, released under GPL.
- */
+/* eb.h, edbrowse header file. */
 
 #ifndef EB_H
 #define EB_H 1
@@ -281,7 +278,7 @@ struct DBTABLE {
 extern CURL *global_http_handle;
 extern CURLSH *global_share_handle;
 extern int debugLevel;		/* 0 to 9 */
-extern bool debugClone, debugEvent, debugThrow, debugCSS;
+extern bool debugClone, debugEvent, debugThrow, debugCSS, dhs;
 extern bool demin; // deminimize javascript
 extern bool uvw; // trace points
 extern bool gotimers; // run javascript timers
@@ -289,7 +286,8 @@ extern int timerspeed; // slowdown factor for javascript timers
 extern int rr_interval; // rerender the screen after this many seconds
 extern FILE *debugFile;
 extern char *debugFileName;
-extern char *sslCerts;		/* ssl certificates to validate the secure server */
+extern char *sslCerts;		// ssl certificates to validate the secure server
+extern char *pubKey;		// publick key file for ssh etc
 extern int verifyCertificates;	/* is a certificate required for the ssl connection? */
 extern int displayLength;	// when printing a line
 extern int formatLineLength;	// when formatting html
@@ -298,23 +296,22 @@ extern int webTimeout, mailTimeout;
 extern uchar browseLocal;
 extern bool sqlPresent;		/* Was edbrowse compiled with SQL built in? */
 extern bool curlActive; // is curl running?
-extern bool ismc;		/* Is the program running as a mail client? */
-extern bool isimap;		/* Is the program running as an imap client? */
-// next two variables work around curl bug 7284
+extern bool ismc;		// Is the program running as a mail client?
+extern bool isimap;		// Is the program running as an imap client?
+extern bool passMail;		// pass mail across the filters
 extern bool down_bg;		// download in background
 extern bool down_jsbg;		// download javascript in background
 extern bool re_utf8, endMarks;
 extern char showProgress; // feedback as a file is downloaded
 extern char selsep; // selection separator
-extern char eb_language[];		/* edbrowse language, determined by $LANG */
+extern char eb_language[];		// edbrowse language, determined by $LANG
 extern  const char *const supported_languages[];
 extern int eb_lang; // encoded version of the above, for languages that we recognize
-extern bool cons_utf8;		/* does the console expect utf8? */
-extern bool iuConvert;		/* perform iso utf8 conversions automatically */
-extern char type8859;		/* 1 through 15 */
-extern bool js_redirects;	/* window.location = new_url */
-extern bool passMail;		/* pass mail across the filters */
-extern bool errorExit;		/* exit on any error, for scripting purposes */
+extern bool cons_utf8;		// does the console expect utf8?
+extern bool iuConvert;		// perform iso utf8 conversions automatically
+extern char type8859;		// 1 through 15
+extern bool js_redirects;	// window.location = new_url
+extern bool errorExit;		// exit on any error, for scripting purposes
 extern bool isInteractive;
 extern volatile bool intFlag;	/* set this when interrupt signal is caught */
 extern time_t intStart;
@@ -340,12 +337,12 @@ extern char *ebUserDir;		/* $ebTempDir/nnn user ID appended */
 extern char *dbarea, *dblogin, *dbpw;	/* to log into the database */
 extern bool fetchBlobColumns, ls_reverse;
 extern uchar ls_sort, dirWrite;
-extern bool caseInsensitive, searchStringsAll, searchWrap;
+extern char lsformat[12];
+extern bool caseInsensitive, searchStringsAll, searchWrap, ebre;
 extern bool allowRedirection;	/* from http code 301, or http refresh */
 extern bool sendReferrer;	/* in the http header */
-extern bool allowJS;		/* javascript on */
+extern bool allowJS;		// javascript on
 extern bool blockJS; // javascript is blocked
-extern bool htmlGenerated;
 extern bool ftpActive;
 extern bool helpMessagesOn;	/* no need to type h */
 extern bool pluginsOn;		/* plugins are active */
@@ -529,7 +526,7 @@ extern struct ebSession sessionList[];
 extern struct ebSession *cs;	/* current session */
 extern int maxSession;
 
-/* The information on an html tag */
+// The information on an html tag
 #define MAXTAGNAME 20
 struct tagInfo {
 	const char name[MAXTAGNAME];
@@ -540,25 +537,23 @@ struct tagInfo {
 };
 extern const struct tagInfo availableTags[];
 
-/* Information on tagInfo->bits */
-/* support innerHTML */
+// Information on tagInfo->bits
+// support innerHTML
 #define TAG_INNERHTML 1
-/* You won't see the text between <foo> and </fooo> */
+// You won't see the text between <foo> and </fooo>
 #define TAG_INVISIBLE 2
-/* sometimes </foo> means nothing. */
+// sometimes </foo> means nothing.
 #define TAG_NOSLASH 4
 
 /* The structure for an html tag.
  * These tags are at times linked with js objects,
  * or even created by js objects. */
 struct htmlTag {
-/* maintain a tree structure */
+// maintain a tree structure
 	struct htmlTag *parent, *firstchild, *sibling;
-/* connect <foo> and </foo> */
-	struct htmlTag *balance;
 	struct htmlTag *same; // same action
-	struct ebFrame *f0; /* frame that owns this tag */
-	struct ebFrame *f1; /* subordinate frame if this is a <frame> tag */
+	struct ebFrame *f0; // frame that owns this tag
+	struct ebFrame *f1; // subordinate frame if this is a <frame> tag
 	jsobjtype jv;		// javascript value
 	int seqno; // tag sequence number in this window
 	int gsn; // global sequence number, for rooting
@@ -634,6 +629,7 @@ struct htmlTag {
 };
 
 typedef struct htmlTag Tag;
+extern Tag *innerParent;
 
 /* htmlTag.action */
 enum {
@@ -654,9 +650,7 @@ enum {
 	TAGACT_OBJ, TAGACT_DATAL, TAGACT_UNKNOWN,
 };
 
-/* htmlTag.itype */
-/* Warning - the order of these is important! */
-/* Corresponds to inp_types in decorate.c */
+// htmlTag.itype, The order of these is important!
 enum {
 	INP_RESET, INP_BUTTON, INP_IMAGE, INP_SUBMIT,
 	INP_HIDDEN, INP_TEXT, INP_FILE,
