@@ -10,7 +10,8 @@ we need to indicate that all the other functions are straight C.
 extern "C" {
 #endif
 
-/* sourcefile=buffers.c */
+// sourcefile=buffers.c
+void setHistcontrol(void);
 void undoSpecialClear(void);
 void removeHiddenNumbers(pst p, uchar terminate);
 pst fetchLine(int n, int show);
@@ -24,6 +25,7 @@ void cxSwitch(int cx, bool interactive) ;
 bool addTextToBuffer(const pst inbuf, int length, int destl, bool showtrail) ;
 void delText(int start, int end) ;
 bool readFileArgv(const char *filename, int fromframe, const char *orig_head);
+bool writeFile(const char *name, int mode);
 Tag *line2frame(int ln);
 bool unfoldBufferW(const struct ebWindow *w, bool cr, char **data, int *len);
 bool unfoldBuffer(int cx, bool cr, char **data, int *len);
@@ -33,29 +35,29 @@ bool edbrowseCommand(const char *line, bool script) ;
 int sideBuffer(int cx, const char *text, int textlen, const char *bufname);
 void freeEmptySideBuffer(int n);
 bool browseCurrentBuffer(void) ;
-bool locateTagInBuffer(int tagno, int *ln_p, char **p_p, char **s_p, char **t_p);
-char *getFieldFromBuffer(int tagno);
+bool locateTagInBuffer(int tagno, int *ln1_p, int *ln2_p, char **p1_p, char **p2_p, char **s_p, char **t_p);
+char *getFieldFromBuffer(int tagno, int ln0);
 int fieldIsChecked(int tagno);
 
-/* sourcefile=dbodbc.c (and others) */
+// sourcefile=dbodbc.c (and others)
 bool fetchForeign(char *tname) ;
 
-/* sourcefile=dbops.c */
+// sourcefile=dbops.c
 int findColByName(const char *name) ;
-
-/* sourcefile=dbstubs.c */
 bool sqlReadRows(const char *filename, char **bufptr) ;
 void dbClose(void) ;
+int unfoldRowCheck(int ln);
+void sql_unfold(int start, int end, char action);
 void showColumns(void) ;
 void showForeign(void) ;
 bool showTables(void) ;
 bool sqlDelRows(int start, int end) ;
-bool sqlUpdateRow(pst source, int slen, pst dest, int dlen) ;
+bool sqlUpdateRow(int ln, pst source, int slen, pst dest, int dlen) ;
 bool sqlAddRows(int ln) ;
 bool ebConnect(void) ;
 int goSelect(int *startLine, char **rbuf) ;
 
-/* sourcefile=fetchmail.c */
+// sourcefile=fetchmail.c
 bool setEnvelopeFormat(const char *s);
 int fetchMail(int account);
 int fetchAllMail(void);
@@ -65,7 +67,7 @@ void mail64Error(int err);
 char *emailParse(char *buf);
 bool setupReply(bool all);
 
-/* sourcefile=format.c */
+// sourcefile=format.c
 void prepareForBrowse(char *h, int h_len);
 void prepareForField(char *h);
 bool breakLine(const char *line, int len, int *newlen);
@@ -97,9 +99,8 @@ void clearEmojis(void);
 void loadEmojis(void);
 char *selectEmoji(const char *p, int len);
 
-/* sourcefile=html.c */
+// sourcefile=html.c
 void dwStart(void);
-void set_basehref(const char *b);
 bool tagHandler(int seqno, const char *name);
 void jSyncup(bool fromtimer, const Tag *active);
 void jSideEffects(void);
@@ -112,8 +113,8 @@ void preFormatCheck(int tagno, bool * pretag, bool * slash);
 char *htmlParse(char *buf, int remote);
 bool htmlTest(void);
 void infShow(int tagno, const char *search);
-bool itext(int d);
-bool infReplace(int tagno, const char *newtext, bool notify);
+bool infReplace(int tagno, char *newtext, bool notify);
+char *displayOptions(const Tag *sel);
 bool infPush(int tagno, char **post_string);
 void domSetsTagValue(Tag *t, const char *newtext);
 void domSubmitsForm(Tag *t, bool reset);
@@ -134,38 +135,31 @@ void delTimers(const Frame *f);
 void runTimer(void);
 void showTimers(void);
 void domOpensWindow(const char *href, const char *name);
+char *render(void);
+bool itext(int d);
 struct htmlTag *line2tr(int ln);
 bool showHeaders(int ln);
+void html_from_setter( Tag *innerParent, const char *h);
 
-/* sourcefile=html-tidy.c */
-void html2nodes(const char *htmltext, bool startpage);
-
-/* sourcefile=decorate.c */
-void traverseAll(int start);
+// sourcefile=html-tags.c
+void htmlScanner(const char *htmltext, Tag *above, bool isgen);
+void setTagAttr(Tag *t, const char *name, char *val);
 const char *attribVal(const Tag *t, const char *name);
 bool attribPresent(const Tag *t, const char *name);
-void setTagAttr(Tag *t, const char *name, char *val);
+Tag *newTag(const Frame *f, const char *tagname);
+void freeTags(struct ebWindow *w);
+void initTagArray(void);
+void traverseAll(void);
 Tag *findOpenTag(Tag *t, int action);
+Tag *findOpenSection(Tag *t);
 Tag *findOpenList(Tag *t);
 void formControl(Tag *t, bool namecheck);
 void htmlInputHelper(Tag *t);
-char *displayOptions(const Tag *sel);
-void prerender(int start);
-char *render(int start);
+void prerender(void);
 const char *fakePropName(void);
-void decorate(int start);
-void freeTags(struct ebWindow *w);
-Tag *newTag(const Frame *f, const char *tagname);
-void initTagArray(void);
-void tag_gc(void);
-void tag_gc(void);
-void htmlNodesIntoTree(int start, Tag *attach);
-void underKill(Tag *t);
-void html_from_setter( Tag *innerParent, const char *h);
-void debugGenerated(const char *h);
-bool frameFromContext(jsobjtype cx);
+void decorate(void);
 
-/* sourcefile=http.c */
+// sourcefile=http.c
 void eb_curl_global_init(void);
 void eb_curl_global_cleanup(void);
 size_t eb_curl_callback(char *incoming, size_t size, size_t nitems, struct i_get *g);
@@ -177,6 +171,7 @@ void *httpConnectBack1(void *ptr);
 void *httpConnectBack2(void *ptr);
 void *httpConnectBack3(void *ptr);
 void ebcurl_setError(CURLcode curlret, const char *url, int action, const char *curl_error);
+int ftpWrite(const char *url);
 void setHTTPLanguage(const char *lang);
 int prompt_and_read(int prompt, char *buffer, int buffer_length, int error_message, bool hide_echo);
 int ebcurl_debug_handler(CURL * handle, curl_infotype info_desc, char *data, size_t size, struct i_get *g);
@@ -186,7 +181,7 @@ bool frameExpand(bool expand, int ln1, int ln2);
 int frameExpandLine(int ln, Tag *t);
 bool reexpandFrame(void);
 
-/* sourcefile=main.c */
+// sourcefile=main.c
 void ebClose(int n);
 void setDataSource(char *v);
 bool javaOK(const char *url);
@@ -200,23 +195,16 @@ struct DBTABLE *newTableDescriptor(const char *name);
 bool readConfigFile(void);
 const char *fetchReplace(const char *u);
 
-/* sourcefile=plugin.c */
-const struct MIMETYPE *findMimeByURL(const char *url, uchar *sxfirst);
-const struct MIMETYPE *findMimeByFile(const char *filename);
-const struct MIMETYPE *findMimeByContent(const char *content);
-bool runPluginCommand(const struct MIMETYPE *m, const char *inurl, const char *infile, const char *indata, int inlength, char **outdata, int *outlength);
-int playBuffer(const char *line, const char *playfile);
-
-/* sourcefile=sendmail.c */
+// sourcefile=sendmail.c
 bool loadAddressBook(void);
 const char *reverseAlias(const char *reply);
-bool encodeAttachment(const char *file, int ismail, bool webform, const char **type_p, const char **enc_p, char **data_p);
+bool encodeAttachment(const char *file, int ismail, bool webform, const char **type_p, const char **enc_p, char **data_p, bool *long_p);
 char *makeBoundary(void);
 bool sendMail(int account, const char **recipients, const char *body, int subjat, const char **attachments, const char *refline, int nalt, bool dosig);
 bool validAccount(int n);
 bool sendMailCurrent(int sm_account, bool dosig);
 
-/* sourcefile=messages.c */
+// sourcefile=messages.c
 void eeCheck(void);
 void eb_puts(const char *s);
 void selectLanguage(void); //@ called once at startup
@@ -292,8 +280,8 @@ void debugPrint(int lev, const char *msg, ...) ;
 void setDebugFile(const char *name); //@
 void nl(void) ;
 int perl2c(char *t) ;
-unsigned pstLength(pst s) ;
-pst clonePstring(pst s) ;
+unsigned pstLength(const pst s) ;
+pst clonePstring(const pst s) ;
 void copyPstring(pst s, const pst t) ;
 bool fdIntoMemory(int fd, char **data, int *len) ;
 bool fileIntoMemory(const char *filename, char **data, int *len) ;
@@ -333,7 +321,7 @@ void appendFile(const char *fname, const char *message, ...) ;
 void appendFileNF(const char *filename, const char *msg) ;
 int eb_system(const char *cmd, bool print_on_success);
 
-/* sourcefile=isup.c */
+// sourcefile=isup.c
 void unpercentURL(char *url) ;
 void unpercentString(char *s) ;
 char *percentURL(const char *start, const char *end);
@@ -380,8 +368,13 @@ bool getUserPassRealm(const char *url, char *creds, const char *realm);
 // Add authorization entries only in the foreground, but it's an
 // atomic operation, so the previous routines can run from other threads.
 bool addWebAuthorization(const char *url, const char *credentials, bool proxy, const char *realm);
+const struct MIMETYPE *findMimeByURL(const char *url, uchar *sxfirst);
+const struct MIMETYPE *findMimeByFile(const char *filename);
+const struct MIMETYPE *findMimeByContent(const char *content);
+bool runPluginCommand(const struct MIMETYPE *m, const char *inurl, const char *infile, const char *indata, int inlength, char **outdata, int *outlength);
+int playBuffer(const char *line, const char *playfile);
 
-/* sourcefile=css.c */
+// sourcefile=css.c
 void writeShortCache(void);
 bool matchMedia(char *t);
 void cssDocLoad(int frameNumber, char *s, bool pageload);
@@ -392,7 +385,7 @@ bool querySelector0(const char *selstring, Tag *top);
 void cssApply(int frameNumber, Tag *t, int pe);
 void cssText(const char *rulestring);
 
-/* sourcefile=jseng-* */
+// sourcefile=jseng-quick.c
 void disconnectTagObject(Tag *t);
 void reconnectTagObject(Tag *t);
 bool has_property_t(const Tag *t, const char *name);
@@ -452,6 +445,8 @@ void set_gcs_number(const char *name, int n);
 void set_gcs_bool(const char *name, bool v);
 void set_gcs_string(const char *name, const char *s);
 void jsClose(void);
+void underKill(Tag *t);
+void set_basehref(const char *b);
 void set_location_hash(const char *h);
 
 #ifdef __cplusplus
