@@ -2,7 +2,6 @@
  * Get mail using the pop3 protocol.
  * Format the mail in ascii, or in html.
  * Unpack attachments.
- * This file is part of the edbrowse project, released under GPL.
  */
 
 #include "eb.h"
@@ -12,7 +11,7 @@
 #endif
 
 #define MHLINE 512		// length of a mail header line
-/* headers and other information about an email */
+// headers and other information about an email
 struct MHINFO {
 	struct MHINFO *next, *prev;
 	struct listHead components;
@@ -24,25 +23,27 @@ struct MHINFO {
 	char date[MHLINE];
 	char boundary[MHLINE];
 	int boundlen;
-/* recipients and cc recipients */
+// recipients and cc recipients
 	char *tolist, *cclist;
 	int tolen, cclen;
-	char mid[MHLINE];	/* message id */
-	char ref[MHLINE];	/* references */
-	char cfn[MHLINE];	/* content file name */
-	uchar ct, ce;		/* content type, content encoding */
+	char mid[MHLINE];	// message id
+	char ref[MHLINE];	// references
+	char cfn[MHLINE];	// content file name
+	uchar ct, ce;		// content type, content encoding
 	bool andOthers;
 	bool doAttach;
 	bool atimage;
 	bool pgp;
+	bool dispat;
 	uchar error64;
 };
 
-static int nattach;		/* number of attachments */
-static int nimages;		/* number of attached images */
-static char *firstAttach;	/* name of first file */
+static int nattach;		// number of attachments
+static int nimages;		// number of attached images
+static char *firstAttach;	// name of first file
 static bool mailIsHtml;
-static char *fm;		/* formatted mail string */
+static bool preferPlain;
+static char *fm;		// formatted mail string
 static int fm_l;
 static struct MHINFO *lastMailInfo;
 static char *lastMailText;
@@ -712,9 +713,10 @@ reaction:
 action:
 		delflag = false;
 		postkey = 0;
+		preferPlain = false;
 		printf("? ");
 		fflush(stdout);
-		key = getLetter("h?qvbefdslmnp gwWuUa/");
+		key = getLetter("h?qvbefdslmnp gtwWuUa/");
 		printf("\b\b\b");
 		fflush(stdout);
 		if (key == '?' || key == 'h') {
@@ -746,7 +748,9 @@ imap_done:
 		if(strchr("wWuUa", key))
 			postkey = key, key = ' ';
 
-		if (key == ' ' || key == 'g') {
+dispmail:
+		if (key == ' ' || key == 'g' || key == 't') {
+			if(key == 't') preferPlain = true;
 /* download the email from the imap server */
 			sprintf(cust_cmd, "FETCH %d BODY[]", mif->seqno);
 			curl_easy_setopt(handle, CURLOPT_CUSTOMREQUEST,
@@ -805,6 +809,7 @@ You'll see this after the perform function runs.
 			key = presentMail();
 /* presentMail has already freed mailstring */
 			postkey = 0;
+			if(key == 'g') goto dispmail;
 		}
 
 		if (key == 'p') {
@@ -1679,7 +1684,7 @@ int fetchMail(int account)
 		if (res != CURLE_OK)
 			goto fetchmail_cleanup;
 		nzFree(message_url);
-		message_url = NULL;
+		message_url = 0;
 	}
 
 fetchmail_cleanup:
@@ -1690,6 +1695,7 @@ fetchmail_cleanup:
 	curl_easy_cleanup(mail_handle);
 	nzFree(message_url);
 	nzFree(mailbox_url);
+	message_url = mailbox_url = 0;
 	nzFree(mailstring);
 	mailstring = initString(&mailstring_l);
 	return nfetch;
@@ -1731,6 +1737,7 @@ static void writeReplyInfo(const char *addstring);
 void scanMail(void)
 {
 	int nmsgs, m;
+	char key;
 
 	if (!isInteractive)
 		i_printfExit(MSG_FetchNotBackgnd);
@@ -1757,6 +1764,7 @@ void scanMail(void)
 
 	loadAddressBook();
 
+	preferPlain = false;
 	for (m = 1; m <= nmsgs; ++m) {
 		nzFree(lastMailText);
 		lastMailText = 0;
@@ -1767,9 +1775,13 @@ void scanMail(void)
 			showErrorAbort();
 		unreadBase = unreadMin;
 
-		if (presentMail() == 'd')
+		key = presentMail();
+		if(key == 'g') { --m, --unreadBase; continue; }
+		if(key == 't') { preferPlain ^= 1; --m, --unreadBase; continue; }
+		if (key == 'd')
 			unlink(umf);
-	}			/* loop over mail messages */
+		preferPlain = false;
+	}			// loop over mail messages
 
 	exit(0);
 }
@@ -1865,7 +1877,7 @@ key_command:
 /* interactive prompt depends on whether there is more text or not */
 		printf("%c ", displine > cw->dol ? '?' : '*');
 		fflush(stdout);
-		key = getLetter((isimap ? "qvbfh? npwWuUasdm" : "qh? nwud"));
+		key = getLetter((isimap ? "qvbfh? gtnpwWuUasdm" : "qh? gtnwud"));
 		printf("\b\b\b");
 		fflush(stdout);
 	}
@@ -1879,6 +1891,16 @@ key_command:
 		i_puts(MSG_Next);
 		goto afterinput;
 
+	case 'g':
+		i_puts(MSG_Restart);
+		goto afterinput;
+
+	case 't':
+		i_puts(MSG_Restart);
+		preferPlain ^= 1;
+		key = 'g';
+		goto afterinput;
+
 	case 'p':
 		i_puts(MSG_Previous);
 		goto afterinput;
@@ -2052,7 +2074,7 @@ afterinput:
 
 	if (delflag)
 		return 'd';
-	return strchr("smvbfp", key) ? key : 'n';
+	return strchr("smvbfpg", key) ? key : 'n';
 }
 
 /* Here are the common keywords for mail header lines.
@@ -2200,14 +2222,15 @@ static void ctExtras(struct MHINFO *w, const char *s, const char *t)
 	if (w->ct < CT_MULTI) {
 		quote = 0;
 		for (q = s + 1; q < t; ++q) {
-			if (isalnumByte(q[-1]))
-				continue;
-/* could be name= or filename= */
-			if (memEqualCI(q, "file", 4))
-				q += 4;
-			if (!memEqualCI(q, "name=", 5))
-				continue;
-			q += 5;
+			if (isalnumByte(q[-1])) continue;
+// could be name= or filename=
+			if (memEqualCI(q, "file", 4)) q += 4;
+			if (!memEqualCI(q, "name", 4)) continue;
+			q += 4;
+			while(*q == ' ') ++q;
+			if(*q != '=') continue;
+			++q;
+			while(*q == ' ') ++q;
 			if (*q == '"') {
 				quote = *q;
 				++q;
@@ -2234,15 +2257,17 @@ static void ctExtras(struct MHINFO *w, const char *s, const char *t)
 			break;
 		}
 	}
-	/* regular file */
+
 	if (w->ct >= CT_MULTI) {
 		quote = 0;
 		for (q = s + 1; q < t; ++q) {
-			if (isalnumByte(q[-1]))
-				continue;
-			if (!memEqualCI(q, "boundary=", 9))
-				continue;
-			q += 9;
+			if (isalnumByte(q[-1])) continue;
+			if (!memEqualCI(q, "boundary", 8)) continue;
+			q += 8;
+			while(*q == ' ') ++q;
+			if(*q != '=') continue;
+			++q;
+			while(*q == ' ') ++q;
 			if (*q == '"') {
 				quote = *q;
 				++q;
@@ -2390,7 +2415,7 @@ static void extractLessGreater(char *s)
  * glean from the headers.
  * Returns a pointer to an allocated MHINFO structure.
  * This routine is recursive. */
-static struct MHINFO *headerGlean(char *start, char *end)
+static struct MHINFO *headerGlean(char *start, char *end, bool top)
 {
 	char *s, *t, *q;
 	char *vl, *vr;		/* value left and value right */
@@ -2478,8 +2503,7 @@ static struct MHINFO *headerGlean(char *start, char *end)
 				--q;	/* try again */
 			}
 			strncpy(w->subject, vl, vr - vl);
-			vl = w->subject;
-			mhReformat(vl);
+			mhReformat((vl = w->subject));
 			vr = vl + strlen(vl);
 			isoDecode(vl, &vr);
 			*vr = 0;
@@ -2589,11 +2613,20 @@ static struct MHINFO *headerGlean(char *start, char *end)
 				w->ct = CT_MULTI;
 			if (memEqualCI(vl, "multipart/alternative", 21))
 				w->ct = CT_ALT;
+			if (memEqualCI(vl, "multipart/related", 17))
+				w->ct = CT_ALT;
 
 			ctExtras(w, s, t);
 			continue;
 		}
 
+		if (memEqualCI(s, "content-disposition:", q - s)) {
+			if (memEqualCI(vl, "attachment", 10))
+				w->dispat = true;
+//			ctExtras(w, s, t);
+			continue;
+		}
+
 		if (memEqualCI(s, "content-transfer-encoding:", q - s)) {
 			if (memEqualCI(vl, "quoted-printable", 16))
 				w->ce = CE_QP;
@@ -2666,9 +2699,20 @@ static struct MHINFO *headerGlean(char *start, char *end)
 		printf("message: %s\n", w->mid);
 		printf("boundary: %d|%s\n", w->boundlen, w->boundary);
 		printf("filename: %s\n", w->cfn);
+		printf("length %d\n", w->end - w->start);
 		printf("content %d/%d\n", w->ct, w->ce);
 	}
 
+// look for line of equals separating successive emails in the same file.
+// Yeah, a false positive is possible here.
+// Fortunately, most emails, even those generated by edbrowse, are multipart,
+// and won't fall into this code.
+	if(top && w->ct < CT_MULTI) {
+		t = strstr(w->start, "\n======================================================================\n");
+		if(t && t < w->end) w->end = t + 1;
+	}
+
+
 	if (w->ce == CE_QP)
 		unpackQP(w);
 	if (w->ce == CE_64) {
@@ -2676,26 +2720,21 @@ static struct MHINFO *headerGlean(char *start, char *end)
 		if (w->error64 != GOOD_BASE64_DECODE)
 			mail64Error(w->error64);
 	}
-	if ((w->ce == CE_64 && w->ct == CT_OTHER) ||
-	    w->ct == CT_APPLIC || w->cfn[0]) {
-		w->doAttach = true;
-		++nattach;
-		q = w->cfn;
-		if (*q) {	/* name present */
+	if(w->ct < CT_MULTI &&
+	(w->dispat || w->ct == CT_OTHER || w->ct == CT_APPLIC)) {
+		w->doAttach = true, ++nattach;
+		if(*(q = w->cfn)) { // name present
 			if (stringEqual(q, "winmail.dat")) {
-				w->atimage = true;
-				++nimages;
+				w->atimage = true, ++nimages;
 			} else if ((q = strrchr(q, '.'))) {
 				static const char *const imagelist[] = {
 					"gif", "jpg", "tif", "bmp", "asc",
 					"png", 0
 				};
-/* the asc isn't an image, it's a signature card. */
-/* Similarly for the winmail.dat */
-				if (stringInListCI(imagelist, q + 1) >= 0) {
-					w->atimage = true;
-					++nimages;
-				}
+// the asc isn't an image, it's a signature card.
+// Similarly for the winmail.dat
+				if (stringInListCI(imagelist, q + 1) >= 0)
+					w->atimage = true, ++nimages;
 			}
 			if (!w->atimage && nattach == nimages + 1)
 				firstAttach = w->cfn;
@@ -2704,7 +2743,7 @@ static struct MHINFO *headerGlean(char *start, char *end)
 	}
 
 /* loop over the mime components */
-	if (w->ct == CT_MULTI || w->ct == CT_ALT) {
+	if (w->ct >= CT_MULTI) {
 		char *lastbound = 0;
 		bool endmode = false;
 		struct MHINFO *child;
@@ -2724,7 +2763,7 @@ static struct MHINFO *headerGlean(char *start, char *end)
 			debugPrint(5, "boundary found at offset %d",
 				   t - w->start);
 			if (lastbound) {
-				child = headerGlean(lastbound, t);
+				child = headerGlean(lastbound, t, false);
 				addToListBack(&w->components, child);
 			}
 			s = lastbound = q;
@@ -2733,8 +2772,9 @@ static struct MHINFO *headerGlean(char *start, char *end)
 		return w;
 	}
 
-	/* mime or alt */
-	/* Scan through, we might have a mail message included inline */
+#if 0
+	// Scan through, we might have a mail message included inline
+// Trying to pull mime components out of my ass? What was I thinking?
 	vl = 0;			/* first mail header keyword line */
 	for (s = start; s < end; s = t + 1) {
 		char first = *s;
@@ -2746,7 +2786,7 @@ static struct MHINFO *headerGlean(char *start, char *end)
 				continue;
 /* Do we have enough for a mail header? */
 			if (k >= 4 && k * 2 >= j) {
-				struct MHINFO *child = headerGlean(vl, end);
+				struct MHINFO *child = headerGlean(vl, end, false);
 				addToListBack(&w->components, child);
 				w->end = end = vl;
 				goto textonly;
@@ -2775,18 +2815,19 @@ static struct MHINFO *headerGlean(char *start, char *end)
 
 /* Header could be at the very end */
 	if (vl && k >= 4 && k * 2 >= j) {
-		struct MHINFO *child = headerGlean(vl, end);
+		struct MHINFO *child = headerGlean(vl, end, false);
 		addToListBack(&w->components, child);
 		w->end = end = vl;
 	}
+#endif
 
 textonly:
 /* Any additional processing of the text, from start to end, can go here. */
 /* Remove leading blank lines or lines with useless words */
 	for (s = start; s < end; s = t + 1) {
 		t = strchr(s, '\n');
-		if (!t)
-			t = end - 1;	/* should never happen */
+		if (!t || t > end)
+			t = end;
 		vl = s, vr = t;
 		if (vr - vl >= 4 && memEqualCI(vr - 4, "<br>", 4))
 			vr -= 4;
@@ -2806,8 +2847,10 @@ textonly:
 			continue;
 		if (memEqualCI(vl, "original message", vr - vl))
 			continue;
-		break;		/* something real */
+		break;		// something real
 	}
+// s could be end + 1
+	if(s > w->end) s = w->end;
 	w->start = start = s;
 
 	return w;
@@ -2872,9 +2915,13 @@ static char *headerShow(struct MHINFO *w, bool top)
 		}
 	} else {
 
-/* This is at the top of the file */
+// This is at the top of the file
 		if (w->subject[0]) {
-			sprintf(buf, "Subject: %s\n", w->subject);
+// the <pre nowspc keeps the subject on one line
+			sprintf(buf, "%sSubject: %s%s\n",
+			(mailIsHtml ? "<pre nowspc>" : ""),
+			w->subject,
+			(mailIsHtml ? "</pre>" : ""));
 			lines = true;
 		}
 		if (nattach && ismc) {
@@ -2918,8 +2965,11 @@ static char *headerShow(struct MHINFO *w, bool top)
 			if (lines & mailIsHtml)
 				strcat(buf, "<br>");
 			lines = true;
+// it's very rare for a from line to break; you have to have a small fll value.
+			if(mailIsHtml) strcat(buf, "<pre nowspc>");
 			strcat(buf, "From ");
 			strcat(buf, w->from);
+			if(mailIsHtml) strcat(buf, "</pre>");
 			strcat(buf, "\n");
 		}
 		if (w->date[0] && !ismc) {
@@ -2955,31 +3005,27 @@ static int mailTextType(struct MHINFO *w)
 	if (w->doAttach)
 		return CT_OTHER;
 
-/* jump right into the hard part, multi/alt */
+// jump right into the hard part, multi/alt
 	if (w->ct >= CT_MULTI) {
 		foreach(v, w->components) {
 			rc = mailTextType(v);
-			if (rc == CT_HTML)
-				return rc;
-			if (rc == CT_OTHER)
-				continue;
-			if (w->ct == CT_MULTI)
-				return rc;
-			texttype = rc;
+			if(!rc) continue;
+			if(!texttype) { texttype = rc; continue; }
+			if(rc == texttype) continue;
+			if(preferPlain ^ (rc == CT_HTML)) texttype = rc;
 		}
 		return texttype;
 	}
 
-	/* multi */
-	/* If there is no text, return nothing */
+	// If there is no text, return nothing
 	if (w->start == w->end)
 		return CT_OTHER;
-/* I don't know if this is right, but I override the type,
- * and make it html, if we start out with <html> */
+// I don't know if this is right, but I override the type,
+// and make it html if we start out with <html>
 	if (memEqualCI(w->start, "<html>", 6))
 		return CT_HTML;
-	return w->ct == CT_HTML ? CT_HTML : CT_TEXT;
-}				/* mailTextType */
+	return w->ct;
+}
 
 static void formatMail(struct MHINFO *w, bool top)
 {
@@ -3011,10 +3057,9 @@ static void formatMail(struct MHINFO *w, bool top)
 			stringAndBytes(&fm, &fm_l, start, end - start);
 			if (mailIsHtml && ct != CT_HTML)
 				stringAndString(&fm, &fm_l, "</pre>\n");
-		}
+		} // text present
 
-		/* text present */
-		/* There could be a mail message inline */
+		// There could be a mail message inline
 		foreach(v, w->components) {
 			if (end > start)
 				stringAndString(&fm, &fm_l,
@@ -3025,13 +3070,13 @@ static void formatMail(struct MHINFO *w, bool top)
 		return;
 	}
 
-	if (ct == CT_MULTI) {
-		foreach(v, w->components)
-		    formatMail(v, false);
-		return;
-	}
-
-/* alternate presentations, pick the best one */
+// alternate presentations, pick the best one.
+// Even if it is generically multipart, I go down this path,
+// cause I don't know what to do.
+// I can't present plain components if we are in html mode,
+// nor html components in plain text mode.
+// This algorithm isn't right, at many levels, but seems to work for
+// virtually every email out there.
 	best = j = 0;
 	foreach(v, w->components) {
 		int subtype = mailTextType(v);
@@ -3053,7 +3098,7 @@ static void formatMail(struct MHINFO *w, bool top)
 		formatMail(v, false);
 		break;
 	}
-}				/* formatMail */
+}
 
 /* Browse the email file. */
 char *emailParse(char *buf)
@@ -3063,8 +3108,9 @@ char *emailParse(char *buf)
 	firstAttach = 0;
 	mailIsHtml = ignoreImages = false;
 	fm = initString(&fm_l);
-	w = headerGlean(buf, buf + strlen(buf));
+	w = headerGlean(buf, buf + strlen(buf), true);
 	mailIsHtml = (mailTextType(w) == CT_HTML);
+	if(mailIsHtml & preferPlain) i_puts(MSG_NoPlain);
 	if (mailIsHtml)
 		stringAndString(&fm, &fm_l, "<html>\n");
 	formatMail(w, true);
@@ -3087,7 +3133,7 @@ char *emailParse(char *buf)
 		lastMailText = buf;
 	}
 	return fm;
-}				/* emailParse */
+}
 
 /*********************************************************************
 Set up for a reply.
@@ -3288,7 +3334,7 @@ nextline:
 		rc = addTextToBuffer((unsigned char *)out, j, 1, false);
 	nzFree(out);
 	return rc;
-}				/* setupReply */
+}
 
 static void writeReplyInfo(const char *addstring)
 {
@@ -3300,7 +3346,7 @@ static void writeReplyInfo(const char *addstring)
 	write(rfh, cw->mailInfo, strlen(cw->mailInfo));
 	write(rfh, "\n", 1);
 	close(rfh);
-}				/* writeReplyInfo */
+}
 
 static void readReplyInfo(void)
 {
@@ -3354,4 +3400,4 @@ found:
 	}
 
 	nzFree(buf);
-}				/* readReplyInfo */
+}
