@@ -1,7 +1,4 @@
-/* format.c
- * Format text, establish line breaks, manage whitespace.
- * This file is part of the edbrowse project, released under GPL.
- */
+/* format.c, Format text, establish line breaks, manage whitespace. */
 
 #include "eb.h"
 
@@ -41,51 +38,63 @@ void prepareForBrowse(char *h, int h_len)
 	h[j] = 0;
 }
 
-/* An input field cannot contain newline, null, or the InternalCodeChar */
-// Revised June 2018, maybe newline is ok. We need it for textarea.
+/* An input field cannot contain cr, lf, null, or the InternalCodeChar */
 void prepareForField(char *h)
 {
 	while (*h) {
-		if (*h == 0)
-			*h = ' ';
-		if (*h == InternalCodeChar)
-			*h = InternalCodeCharAlternate;
+// this line is goofy cause null would terminate the loop
+		if (*h == 0) *h = ' ';
+		if (*h == '\r') *h = '\07';
+		if (*h == InternalCodeChar) *h = InternalCodeCharAlternate;
 		++h;
 	}
 }
 
 /*********************************************************************
-The primary goal of this routine is to turn
-Hey,{ click here } for more information
-into
-Hey, {click here}  for more information
-But of course we won't do that if the section is preformatted.
-Nor can we muck with the whitespace that might be present in an input field <>.
-Also swap 32* whitespace, pushing invisible anchors forward.
-If a change is made, the procedure is run again,
-kinda like bubble sort.
-It has the potential to be terribly inefficient,
-but that doesn't seem to happen in practice.
-Use cnt to count the iterations, just for debugging.
-| is considered a whitespace character. Why is that?
-Html tables are mostly used for visual layout, but sometimes not.
-I use | to separate the cells of a table, but if there's nothing in them,
-or at least no text, then I get rid of the pipes.
-But every cell is going to have an invisible anchor from <td>, so that js can,
-perhaps, set innerHTML inside this cell.
-So there's something there, but nothing there.
-I push these tags past pipes, so I can clear it all away.
-One web page in ten thousand will actually set html inside a cell,
-after the fact, and when that happens the text won't be in the right place,
-it won't have the pipes around it that it should.
-I'm willing to accept that for now.
+The characters 03 and 04 delimite the cells of a table, or data, respectively.
+We often don't know, so fall back to table.
+And tables are often used to format a page, not a real table.  Ugh!
+This routine turns either of these into |
+However if there is one such | on a line, and it is a table cell marker,
+we remove it. Most of the time the table is page layout,
+and the | would only confuse things.
+Remove whitespace before or after <td>, as tidy does.
 *********************************************************************/
 
 static void cellDelimiters(char *buf)
 {
 	char *lastcell = 0;
 	int cellcount = 0;
-	char *s;
+	char *s, *t;
+
+	for (s = t = buf; *s; ++s) {
+		int n;
+		char *u;
+		if(*s != DataCellChar && *s != TableCellChar) {
+			*t++ = *s;
+			continue;
+		}
+// spaces behind
+		while(t > buf && t[-1] == ' ') --t;
+		*t++ = *s; // cell marker
+// spaces ahead
+respace:
+		if(s[1] == ' ') { ++s; goto respace; }
+// tidy turns <td> <i> hello </i> </td> into <td><i>hello</i></td>
+// But not so with <p> or other strong tags.
+// I try to do the same.
+		if(s[1] != InternalCodeChar || !isdigit(s[2])) continue;
+			n = strtol(s + 2, &u, 10);
+// leave input fields alone
+			if(*u != '*' && *u != '{') continue;
+			if(tagList[n]->info->para & 3) continue;
+// looks like a soft tag
+			memcpy(t, s + 1, u - s);
+			t += u - s;
+			s = u;
+			goto respace;
+	}
+	*t = 0;
 
 	for (s = buf; *s; ++s) {
 		if (*s == DataCellChar) {
@@ -100,41 +109,38 @@ static void cellDelimiters(char *buf)
 		}
 		if (!strchr("\f\r\n", *s))
 			continue;
-/* newline here, if just one cell delimiter then blank it out */
+// newline here, if just one cell delimiter then blank it out
 		if (cellcount == 1)
 			*lastcell = ' ';
 		cellcount = 0;
 	}
-}				/* cellDelimiters */
+}
 
-static void anchorSwap(char *buf)
+/*********************************************************************
+this is transliteration, like /bin/tr
+If the output char is x, it goes away, like tr -d
+Some characters, like unicode A0, turn into space,
+so we need to do this now, before swapping anchors and whitespace.
+Watch out for utf8 - don't translate the a0 in c3a0.  That is a grave.
+But a0 by itself is breakspace; turn it into space.
+And c2a0 is utf8 breakspace.
+99.9% of the time we are in utf8 mode.
+I strip out the private utf8 codes.
+Those should never be used on a public website.
+If they are used by your private work website, and you need to retain them,
+export EB_PUA=on
+Don't do any of these transliterations in an input field.
+Those must be exactly preserved, obviously.
+*********************************************************************/
+
+static void html_tr(char *buf)
 {
-	char c, d, *s, *ss, *w, *a;
-	bool pretag;		// <pre>
-	bool premode;		// inside <pre> </pre>
-	bool inputmode;		// inside an input field
-	bool slash;		// closing tag
-	bool change;		// made a swap somewhere
-	bool strong;		// strong whitespace, newline or paragraph
-	int n, cnt;
-	char tag[20];
-	const char *pua; // for private use area
-
-// this is transliteration, like /bin/tr
-// I use to convert a6 and c2 to hyphen space, not sure why
+	char *s, *ss, *w, c, d;
+	int n;
 	static const char from[] = "\x1b\x95\x99\x9c\x9d\x91\x92\x93\x94\xa0\xad\x96\x97\x85";
-// x goes away, like tr -d
 	static const char becomes[] = "_*'`'`'`' x---";
-
-/* Transliterate a few characters.  One of them is 0xa0 to space,
- * so we need to do this now, before the anchors swap with whitespace.
- * Watch out for utf8 - don't translate the a0 in c3a0.  That is a grave.
- * But a0 by itself is breakspace; turn it into space.
- * And c2a0 is a0 is breakspace.
- * Don't do any of these transliterations in an input field. */
-
-	inputmode = false;
-	pua = getenv("EB_PUA");
+	bool inputmode = false;
+	char *pua = getenv("EB_PUA");
 	if(pua && !*pua) pua = 0;
 
 	for (s = w = buf; (c = *s); ++s) {
@@ -156,7 +162,7 @@ static void anchorSwap(char *buf)
 		if (inputmode)
 			goto put1;
 
-/* utf8 test */
+// utf8 test
 		if ((c & 0xc0) == 0xc0 && (d & 0xc0) == 0x80) {
 			unsigned int uni = 0, ubytes = 0;
 			if ((c & 0x3c) == 0) {
@@ -171,7 +177,7 @@ static void anchorSwap(char *buf)
 					goto put1;
 				}
 			}
-/* copy the utf8 sequence as is */
+// copy the utf8 sequence as is
 			uni = 0;
 			*w++ = c;
 			++s;
@@ -185,20 +191,17 @@ static void anchorSwap(char *buf)
 				c = (char) ((uchar)c >> (1+ubytes));
 				uni |= ((unsigned int)c << (ubytes*6));
 // We can do things with high unicodes here, if we wish.
-
 // Suppresse private unicodes, which shouldn't appear on public websites,
 // and if they do, there isn't a consistent way to read them.
-// set EB_PUA=on if you want to retain them.
 			if(((uni >= 0xe000 && uni < 0xf8ff) ||
 			(uni >= 0xf0000 && uni < 0xffffd) ||
 			(uni >= 0x100000 && uni < 0x10fffd)) &&
 			!pua)
 				w -= ubytes+1;
-
 			continue;
 		}
 
-/* Now assuming iso8859-1, which is practically deprecated */
+// Now assuming iso8859-1, which is practically deprecated
 		ss = strchr(from, c);
 		if (ss) {
 			c = becomes[ss - from];
@@ -206,30 +209,64 @@ static void anchorSwap(char *buf)
 				continue;
 		}
 
-#if 0
-// Should we modify empty anchors in any way?
-		if (c != InternalCodeChar)
-			goto put1;
-		if (!isdigitByte(s[1]))
-			goto put1;
-		for (a = s + 2; isdigitByte(*a); ++a) ;
-		if (*a != '{')
-			goto put1;
-		for (++a; *a == ' '; ++a) ;
-		if (a[0] != InternalCodeChar || a[1] != '0' || a[2] != '}')
-			goto put1;
-// do something with empty {} here.
-// Following code just skips it, but we likely shouldn't do that.
-		s = a + 2;
-		continue;
-#endif
-
 put1:
 		*w++ = c;
 	}
 	*w = 0;
+}
+
+// For <h3><div>hello</div></h3> which should not happen, but does
+static void h3div(char *buf)
+{
+	char *s;
+	bool in_h = false;
+	for(s = buf; *s; ++s) {
+		if(*s == '\f' && s[1] == 'h' &&
+		(uchar)s[2] >= '1' && (uchar)s[2] <= '6' && s[3] == ' ') {
+			in_h = true, s += 3;
+			continue;
+		}
+		if(isspaceByte(*s)) {
+			if(in_h) *s = ' ';
+		} else in_h = false;
+	}
+}
+
+/*********************************************************************
+The primary goal of this routine is to turn
+Hey,{ click here } for more information
+into
+Hey, {click here}  for more information
+But of course we won't do that if the section is preformatted.
+Nor can we muck with the whitespace that might be present in an input field <>.
+Also swap 32* whitespace, pushing invisible anchors forward.
+All this swapping preserves the length of the string.
+If a change is made, the procedure is run again,
+kinda like bubble sort.
+It has the potential to be terribly inefficient,
+but that doesn't seem to happen in practice.
+Use cnt to count the iterations, for debugging purposes.
+| is considered a whitespace character. Why is that?
+Html tables are mostly used for visual layout, but sometimes not.
+I use | to separate the cells of a table, but if there's nothing in them,
+or at least no text, then I get rid of the pipes.
+But every cell is going to have an invisible anchor from <td>, so that js can,
+perhaps, set innerHTML inside this cell.
+So there's something there, but nothing there.
+I push these tags past the pipes, so I can clear it all away.
+*********************************************************************/
+
+static void anchorSwap(char *buf)
+{
+	char c, d, *s, *ss, *w, *a;
+	bool pretag;		// <pre>
+	bool premode;		// inside <pre> </pre>
+	bool slash;		// closing tag
+	bool change;		// made a swap somewhere
+	bool strong;		// strong whitespace, newline or paragraph
+	int n, cnt, tagno;
+	char tag[20];
 
-/* anchor whitespace swap preserves the length of the string */
 	cnt = 0;
 	change = true;
 	while (change) {
@@ -266,8 +303,8 @@ put1:
 /* some conditions that should never happen */
 			if (!isdigitByte(s[1]))
 				goto normalChar;
-			n = strtol(s + 1, &ss, 10);
-			preFormatCheck(n, &pretag, &slash);
+			tagno = strtol(s + 1, &ss, 10);
+			preFormatCheck(tagno, &pretag, &slash);
 			d = *ss;
 			if (!strchr("{}<>*", d))
 				goto normalChar;
@@ -294,19 +331,26 @@ put1:
 
 			if ((d == '*' || d == '{') && !premode)
 				a = s;
+			if(d == '<' && stringEqual(tagList[tagno]->info->name, "button"))
+				a = s;
 			s = ss;
 
 normalChar:
 			w = 0;	/* no more whitespace */
-/* end of loop over the chars in the buffer */
 		}
 /* end of loop making changes */
 	}
 	debugPrint(4, "anchorSwap %d", cnt);
+}
 
 /* Framing characters like [] around an anchor are unnecessary here,
  * because we already frame it in braces.
  * Get rid of these characters, even in premode. */
+static void anchorUnframe(char *buf)
+{
+	char c, d, *s, *w, *a;
+	int n;
+
 	for (s = w = buf; (c = *s); ++s) {
 		char open, close, linkchar;
 		if (!strchr("{[(<", c))
@@ -344,7 +388,7 @@ normalChar:
 				++n;
 			d = a[n++];
 			if (!d)
-				break;	/* should never happen */
+				break;	// should never happen
 			if (strchr("{}<>", d))
 				break;
 		}
@@ -362,13 +406,19 @@ normalChar:
 		continue;
 putc:
 		*w++ = c;
-	}			/* loop over buffer */
+	}			// loop over buffer
 	*w = 0;
 	debugPrint(4, "anchors unframed");
+}
+
+// Now compress the implied linebreaks into one.
+static void html_ws(char *buf)
+{
+	char c, d, *s, *w, *a;
+	int n;
+	bool premode = false, pretag, strong, slash;
+	char *ss = 0;
 
-/* Now compress the implied linebreaks into one. */
-	premode = false;
-	ss = 0;
 	for (s = buf; (c = *s); ++s) {
 		if (c == InternalCodeChar && isdigitByte(s[1])) {
 			n = strtol(s + 1, &s, 10);
@@ -380,8 +430,28 @@ putc:
 		}
 		if (!isspaceByte(c))
 			continue;
+
+// whitespace region starts here.
+// Look for pipes inside whitespace, these are usually cell delimiters.
 		strong = false;
-		a = 0;
+		for (w = s; isspaceByte(*w) || *w == '|'; ++w) {
+			if(*w == '|') {
+				if(strong) continue;
+				break;
+			}
+			if(*w != ' ') strong = true;
+		}
+// pipes have to be internal to whitespace to be moved;
+// pipes at the end are left alone.
+		if(*w && *w != InternalCodeChar)
+			while(w[-1] == '|') --w;
+// move pipes to the front
+		for(ss = --w; w >= s; --w)
+			if(*w != '|') *ss-- = *w;
+		while(ss >= s) *ss-- = '|';
+		while(*s == '|') ++s;
+
+		strong = false, a = 0;
 		for (w = s; isspaceByte(*w); ++w) {
 			if (*w == '\n' || *w == '\f')
 				strong = true;
@@ -402,7 +472,7 @@ putc:
 		for (w = ss; w <= s; ++w)
 			if (*w == '\r' && w != a)
 				*w = ' ';
-	}			/* loop over buffer */
+	}			// loop over buffer
 	debugPrint(4, "whitespace combined");
 
 /* Due to the anchor swap, the buffer could end in whitespace
@@ -732,7 +802,7 @@ static int formfeedCount(const char *buf, int len)
 		if (buf[i] == '\f')
 			++ff;
 	return ff;
-}				/* formfeedCount */
+}
 
 bool breakLine(const char *line, int len, int *newlen)
 {
@@ -808,7 +878,7 @@ bool breakLine(const char *line, int len, int *newlen)
 	debugPrint(7, "chunk<EOL>%d.%d", colno, lspace);
 	*newlen = bl_cursor - bl_start;
 	return !bl_overflow;
-}				/* breakLine */
+}
 
 void breakLineSetup(void)
 {
@@ -826,8 +896,11 @@ char *htmlReformat(char *buf)
 	char *fmark;		/* mark the start of a frame */
 
 	cellDelimiters(buf);
-
+	html_tr(buf);
+	h3div(buf);
 	anchorSwap(buf);
+	anchorUnframe(buf);
+	html_ws(buf);
 
 	longcut = lperiod = lcomma = lright = lany = 0;
 	colno = 1;
@@ -926,7 +999,7 @@ char *htmlReformat(char *buf)
 	}
 
 	return new;
-}				/* htmlReformat */
+}
 
 /*********************************************************************
 Crunch a to-list or a copy-to-list down to its email addresses.
@@ -991,7 +1064,7 @@ append:
 			*s = ',';
 	if (*line)
 		strcat(line, ",");
-}				/* extractEmailAddresses */
+}
 
 static void cutDuplicateEmail(char *line, const char *dup, int duplen)
 {
@@ -1007,7 +1080,7 @@ static void cutDuplicateEmail(char *line, const char *dup, int duplen)
 		}
 		line = s + 1;
 	}
-}				/* cutDuplicateEmail */
+}
 
 void cutDuplicateEmails(char *tolist, char *cclist, const char *reply)
 {
@@ -1057,7 +1130,7 @@ void cutDuplicateEmails(char *tolist, char *cclist, const char *reply)
 			cutDuplicateEmail(cclist, r, len);
 		}
 	}
-}				/* cutDuplicateEmails */
+}
 
 bool isEmailAddress(const char *s)
 {
@@ -1103,7 +1176,7 @@ int byteOrderMark(const uchar * buf, int buflen)
 	if (buflen >= 4 && !memcmp(buf, "\x0\x0\xfe\xff", 4))
 		return 6;
 	return 0;
-}				/* byteOrderMark */
+}
 
 /*********************************************************************
 We got some data from a file or from the internet.
@@ -1520,7 +1593,7 @@ void utfHigh(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p,
 	*outbuflen_p = j;
 }
 
-/* convert a 32 bit unicode character into utf8 */
+// convert a 32 bit unicode character into utf8
 char *uni2utf8(unsigned int unichar)
 {
 	static uchar outbuf[12];
@@ -1937,7 +2010,7 @@ bool parseDataURI(const char *uri, char **mediatype, char **data, int *data_l)
 	}
 
 	return true;
-}				/* parseDataURI */
+}
 
 uchar fromHex(char d, char e)
 {
@@ -1948,7 +2021,7 @@ uchar fromHex(char d, char e)
 		e -= ('a' - '9' - 1);
 	d -= '0', e -= '0';
 	return ((((uchar) d) << 4) | (uchar) e);
-}				/* fromHex */
+}
 
 // find the color closest to the rgb value.
 // Input string is allocated; return is either the einput string
