@@ -1,7 +1,4 @@
-/* http.c
- * HTTP protocol client implementation
- * This file is part of the edbrowse project, released under GPL.
- */
+/* http.c,  HTTP protocol client implementation */
 
 #include "eb.h"
 
@@ -130,6 +127,7 @@ static size_t curl_header_callback(char *header_line, size_t size, size_t nmemb,
 				   struct i_get *g);
 static bool ftpConnect(struct i_get *g, char *creds_buf);
 static bool gopherConnect(struct i_get *g);
+static bool dataConnect(struct i_get *g);
 static bool read_credentials(char *buffer);
 static const char *message_for_response_code(int code);
 
@@ -835,6 +833,11 @@ bool httpConnect(struct i_get *g)
 			setError(MSG_DomainEmpty);
 		return false;
 	}
+
+// data is like no other protocol
+	if(stringEqual(prot, "data"))
+		return dataConnect(g);
+
 // plugins can only be ok from one thread, the interactive thread
 // that calls up web pages at the user's behest.
 // None of this machinery need be threadsafe.
@@ -894,9 +897,9 @@ mimestream:
 		return ftpConnect(g, creds_buf);
 	} else {
 		if (g->foreground)
-			setError(MSG_WebProtBad, prot);
+			setError(MSG_WebProtBad, prot, "edbrowse");
 		else if (debugLevel >= 3) {
-			i_printf(MSG_WebProtBad, prot);
+			i_printf(MSG_WebProtBad, prot, "edbrowse");
 			nl();
 		}
 		return false;
@@ -1128,6 +1131,7 @@ So this is a simple workaround.
 		if (g->down_state == 6) {
 // Header has indicated a plugin by content type or protocol or suffix.
 			curl_easy_cleanup(h);
+			mt = cf->mt;
 			goto mimestream;
 		}
 
@@ -1847,7 +1851,7 @@ void ebcurl_setError(CURLcode curlret, const char *url, int action,
 
 	switch (curlret) {
 	case CURLE_UNSUPPORTED_PROTOCOL:
-		(*fn) (MSG_WebProtBad, prot);
+		(*fn) (MSG_WebProtBad, prot, "curl");
 		break;
 
 	case CURLE_URL_MALFORMAT:
@@ -2083,6 +2087,76 @@ ftp_transfer_fail:
 	return transfer_success;
 }
 
+int ftpWrite(const char *url)
+{
+// It's a drag, but we can only upload from a file, not from memory.
+	char *tempfile;
+	static int idx = 0;
+	bool saveChangeMode = cw->changeMode, transfer_success;
+	struct i_get g0;
+	CURL *h = 0;		// the curl handle for ftp
+	CURLcode curlret = CURLE_OK;
+	FILE *f = 0;
+
+	if(url[strlen(url) - 1] == '/') {
+		setError(MSG_NoDir0);
+		return false;
+	}
+
+	if (!ebUserDir) {
+		setError(MSG_TempNone);
+		return false;
+	}
+	++idx;
+	if (asprintf(&tempfile, "%s/ftp%d-%d",
+		     ebUserDir, getpid(), idx) < 0)
+		i_printfExit(MSG_MemAllocError, strlen(ebUserDir) + 24);
+	if(!writeFile(tempfile, 0)) {
+		unlink(tempfile), free(tempfile);
+// keep fileSize as is, in case we wrote part of it
+		return false;
+	}
+
+	memset(&g0, 0, sizeof(g0));
+	g0.url = url;
+	g0.headers = initString(&g0.headers_len);
+	transfer_success = false;
+	if(!(h = http_curl_init(&g0)))
+		goto fail;
+
+/*********************************************************************
+example: https://curl.se/libcurl/c/ftpupload.html
+This does a lot of things that I don't think we need to do.
+On windows you need a callback read function, but edbrowse doesn't run on windows,
+so I'm going to leave that one off.
+Custom headers to upload to a different name, and then rename the local file.
+We don't need to do that either.
+*********************************************************************/
+
+	f = fopen(tempfile, (cw->binMode ? "rb" : "r"));
+	if(!f) { // this should never happen
+		setError(MSG_OpenFail, tempfile);
+		goto fail;
+	}
+	curl_easy_setopt(h, CURLOPT_UPLOAD, 1L);
+	curl_easy_setopt(h, CURLOPT_URL, url);
+	curl_easy_setopt(h, CURLOPT_READDATA, f);
+	curl_easy_setopt(h, CURLOPT_INFILESIZE_LARGE, (curl_off_t)fileSize);
+	curlret = curl_easy_perform(h);
+
+fail:
+	if (h) curl_easy_cleanup(h);
+	if(f) fclose(f);
+	if(curlret == CURLE_OK) transfer_success = true;
+	if (transfer_success == false) {
+		fileSize = -1, cw->changeMode = saveChangeMode;
+			ebcurl_setError(curlret, url, 0, g0.error);
+	}
+	i_get_free(&g0, !transfer_success);
+	unlink(tempfile), free(tempfile);
+	return transfer_success;
+}
+
 /* Like httpConnect, but for gopher */
 static bool gopherConnect(struct i_get *g)
 {
@@ -2223,6 +2297,51 @@ gopher_transfer_fail:
 	return true;
 }
 
+static bool dataConnect(struct i_get *g)
+{
+	char *copy = cloneString(g->url);
+	char *colon = strchr(copy, ':') + 1;
+	char *comma = strchr(copy, ',');
+	debugPrint(3, "data protocol");
+	if(!comma) { // should never happen
+		debugPrint(3, "data has no comma");
+		nzFree(copy);
+		return 0;
+	}
+	*comma++ = 0;
+	unpercentString(comma);
+	if(comma - copy >= 8 && stringEqual(comma - 8, ";base64")) {
+		debugPrint(3, "data:base64");
+		char *end = comma + strlen(comma);
+		if(base64Decode(comma, &end)) {
+			debugPrint(3, "base64 error in data");
+			nzFree(copy);
+			return true;
+		}
+		g->length = end - comma;
+		g->buffer = allocMem(g->length);
+		memcpy(g->buffer, comma, g->length);
+		comma[-8] = 0;
+	} else {
+		g->length = strlen(comma);
+		g->buffer = cloneString(comma);
+	}
+	if(*colon) {
+// pull the type out of the data if we can
+		strncpy(g->content, colon, sizeof(g->content) - 1);
+		caseShift(g->content, 'l');
+		debugPrint(3, "content %s", g->content);
+		g->charset = strchr(g->content, ';');
+		if (g->charset)
+			*(g->charset)++ = 0;
+	}
+	g->hcl = g->length;
+	debugPrint(4, "content length %lld", g->hcl);
+	nzFree(copy);
+	g->cfn = cloneString("data:");
+	return true;
+}
+
 /* If the user has asked for locale-specific responses, then build an
  * appropriate Accept-Language: header. */
 void setHTTPLanguage(const char *lang)
@@ -2282,7 +2401,7 @@ static CURL *http_curl_init(struct i_get *g)
 	curl_init_status = curl_easy_setopt(h, CURLOPT_COOKIEFILE, "");
 	if (curl_init_status != CURLE_OK)
 		goto libcurl_init_fail;
-/* Lots of these setopt calls shouldn't fail.  They just diddle a struct. */
+// Lots of these setopt calls shouldn't fail.  They just diddle a struct.
 	curl_easy_setopt(h, CURLOPT_SOCKOPTFUNCTION, my_curl_safeSocket);
 	curl_easy_setopt(h, CURLOPT_WRITEFUNCTION, eb_curl_callback);
 	curl_easy_setopt(h, CURLOPT_WRITEDATA, g);
@@ -2295,6 +2414,8 @@ static CURL *http_curl_init(struct i_get *g)
 	curl_easy_setopt(h, CURLOPT_NOPROGRESS, 0);
 	curl_easy_setopt(h, CURLOPT_PROGRESSFUNCTION, curl_progress);
 	curl_easy_setopt(h, CURLOPT_PROGRESSDATA, g);
+	if(pubKey)
+		curl_easy_setopt(h, CURLOPT_SSH_PUBLIC_KEYFILE, pubKey);
 	curl_easy_setopt(h, CURLOPT_CONNECTTIMEOUT, webTimeout);
 	curl_easy_setopt(h, CURLOPT_USERAGENT, currentAgent);
 	curl_easy_setopt(h, CURLOPT_SSLVERSION, CURL_SSLVERSION_DEFAULT);
@@ -2557,8 +2678,10 @@ curl_header_callback(char *header_line, size_t size, size_t nmemb,
 
 	if (g->down_ok && g->down_state == 0 &&
 	    !(mt && g->pg_ok && mt->down_url && !mt->from_file) &&
-	    g->content[0] && !memEqualCI(g->content, "text/", 5) &&
-	    !memEqualCI(g->content, "application/xhtml+xml", 21)) {
+// text/whatever can go into the buffer
+	    strlen(g->content) > 4 && !memEqualCI(g->content, "text/", 5) &&
+// whatever+xml can go into the buffer
+	    !memEqualCI(g->content + strlen(g->content) - 4, "+xml", 4)) {
 		g->down_state = 1;
 		g->down_msg = MSG_Down;
 		debugPrint(3, "potential download based on type %s",
@@ -3090,13 +3213,10 @@ So check for serverData null here. Once again we pop the frame.
 	cdt->attributes = allocZeroMem(sizeof(char*));
 	cdt->atvals = allocZeroMem(sizeof(char*));
 	debugPrint(3, "parse html from frame");
-/* call the tidy parser to build the html nodes */
-	html2nodes(serverData, true);
+	htmlScanner(serverData, cdt, false);
 	nzFree(serverData);	/* don't need it any more */
 	serverData = 0;
-	htmlGenerated = false;
-	htmlNodesIntoTree(start + 1, cdt);
-	prerender(0);
+	prerender();
 
 /*********************************************************************
 At this point cdt->step is 1; the html tree is built, but not decorated.
@@ -3105,7 +3225,7 @@ cdt doesn't have or need an object; it's a place holder.
 	cdt->step = 2;
 
 	if (cf->jslink) {
-		decorate(0);
+		decorate();
 		set_basehref(cf->hbase);
 		run_function_bool_win(cf, "eb$qs$start");
 		if (jssrc)
@@ -3212,19 +3332,16 @@ bool reexpandFrame(void)
 
 	start = cw->numTags;
 	debugPrint(3, "parse html from frame replace");
-/* call the tidy parser to build the html nodes */
-	html2nodes(serverData, true);
+	htmlScanner(serverData, cdt, false);
 	nzFree(serverData);	/* don't need it any more */
 	serverData = 0;
 	cf->browseMode = false;
-	htmlGenerated = false;
-	htmlNodesIntoTree(start, cdt);
 	cdt->step = 0;
-	prerender(0);
+	prerender();
 	cdt->step = 2;
 
 	if (cf->jslink) {
-		decorate(0);
+		decorate();
 		set_basehref(cf->hbase);
 		run_function_bool_win(cf, "eb$qs$start");
 		runScriptsPending(true);
