@@ -1,19 +1,19 @@
-/* main.c
- * Entry point, arguments and options.
- * This file is part of the edbrowse project, released under GPL.
- */
+/* main.c, Entry point, arguments and options, config file. */
 
 #include "eb.h"
 
 #include <pthread.h>
 #include <signal.h>
 
-/* Define the globals that are declared in eb.h. */
-/* See eb.h for descriptive comments. */
-
+// Define the globals that are declared in eb.h.
+// See eb.h for additional comments.
+// Let's start with the current (foreground) edbrowse window and frame.
+Window *cw;
+Frame *cf;
+int gfsn; // global frame sequence number
 const char *progname;
 const char eol[] = "\r\n";
-const char *version = "3.8.2.1";
+const char *version = "3.8.3";
 char *changeFileName;
 char *configFile, *addressFile, *cookieFile, *emojiFile;
 char *mailDir, *mailUnread, *mailStash, *mailReply;
@@ -31,7 +31,7 @@ bool ftpActive;
 int webTimeout = 20, mailTimeout = 0;
 int displayLength = 500;
 int verifyCertificates = 1;
-char *sslCerts;
+char *sslCerts, *pubKey;
 int localAccount, maxAccount;
 struct MACCOUNT accounts[MAXACCOUNT];
 int maxMime;
@@ -46,7 +46,7 @@ bool inInput, inInitFunction, listNA;
 int fileSize;
 char *dbarea, *dblogin, *dbpw;	/* to log into the database */
 bool fetchBlobColumns;
-bool caseInsensitive, searchStringsAll, searchWrap = true;
+bool caseInsensitive, searchStringsAll, searchWrap = true, ebre = true;
 bool binaryDetect = true;
 bool inputReadLine;
 bool curlAuthNegotiate = false;
@@ -92,7 +92,7 @@ static void finishBrowse(void)
 	if (cw->browseMode)
 		return;
 // We were in the middle of the browse command; this is typical.
-	a = render(0);
+	a = render();
 	newbuf = htmlReformat(a);
 	nzFree(a);
 	cw->rnlMode = cw->nlMode;
@@ -190,7 +190,8 @@ void ebClose(int n)
 
 static struct ebhost {
 // j = nojs, v = novs, p = proxy, f = function,
-// s = subject, t = to, r = reply, a = agentsite
+// s = subject, t = to, r = reply, a = agentsite,
+// J = js, j = nojs
 	char type, filler;
 	short n;
 // watch out, these fields are highly overloaded, depending on type
@@ -255,6 +256,17 @@ bool javaOK(const char *url)
 		return false;
 	if (isDataURI(url))
 		return true;
+	for (j = 0; (unsigned)j < ebhosts_avail; ++j)
+		if (ebhosts[j].type == 'J' &&
+		    patternMatchURL(url, ebhosts[j].host))
+			return true;
+// inheritance
+	if(cf->hbase && cf->hbase[0]) {
+		for (j = 0; (unsigned)j < ebhosts_avail; ++j)
+			if (ebhosts[j].type == 'J' &&
+			    patternMatchURL(cf->hbase, ebhosts[j].host))
+				return true;
+	}
 	for (j = 0; (unsigned)j < ebhosts_avail; ++j)
 		if (ebhosts[j].type == 'j' &&
 		    patternMatchURL(url, ebhosts[j].host))
@@ -480,7 +492,7 @@ int main(int argc, char **argv)
 	static char agent0[64] = "edbrowse/";
 
 #ifndef _MSC_VER		// port setlinebuf(stdout);, if required...
-/* In case this is being piped over to a synthesizer, or whatever. */
+// In case this is being piped over to a synthesizer, or whatever.
 	if (fileTypeByHandle(fileno(stdout)) != 'f')
 		setlinebuf(stdout);
 #endif // !_MSC_VER
@@ -488,6 +500,9 @@ int main(int argc, char **argv)
 	selectLanguage();
 	setHTTPLanguage(eb_language);
 
+// bring HISTCONTROL down to edbrowse
+	setHistcontrol();
+
 /* Establish the home directory, and standard edbrowse files thereunder. */
 	home = getenv("HOME");
 #ifdef _MSC_VER
@@ -927,11 +942,12 @@ int runEbFunction(const char *line)
 
 // local copies of settings, to restore after function runs.
 	struct {
-		bool rl, endm, lna, H, ci, sg, su8, sw, bd, iu, hf, hr, vs, sr, can, ftpa, bg, jsbg, js, showall, pg, fbc, ls_reverse;
+		bool rl, endm, lna, H, ci, sg, su8, sw, ebre, bd, iu, hf, hr, vs, sr, can, ftpa, bg, jsbg, js, showall, pg, fbc, ls_reverse, fllo;
 		uchar dw, ls_sort;
 		char lsformat[12], showProgress;
 		char *currentAgent;
 		int agentIndex, debugLevel, timerspeed;
+		int ll, fll;
 	} save;
 
 /* Separate function name and arguments */
@@ -974,6 +990,8 @@ int runEbFunction(const char *line)
 		save.timerspeed = timerspeed;
 		save.dw = dirWrite;
 		save.ls_sort = ls_sort;
+		strcpy(save.lsformat, lsformat);
+		save.showProgress = showProgress;
 		save.bg = down_bg;
 		save.jsbg = down_jsbg;
 		save.iu = iuConvert;
@@ -984,6 +1002,7 @@ int runEbFunction(const char *line)
 		save.fbc = fetchBlobColumns;
 		save.ls_reverse = ls_reverse;
 		save.sw = searchWrap;
+		save.ebre = ebre;
 		save.hr = allowRedirection;
 		save.sr = sendReferrer;
 		save.js = allowJS;
@@ -997,6 +1016,9 @@ int runEbFunction(const char *line)
 		save.ci = caseInsensitive;
 		save.su8 = re_utf8;
 		save.vs = verifyCertificates;
+		save.ll = displayLength;
+		save.fll = formatLineLength;
+		save.fllo = formatOverflow;
 	}
 
 /* collect arguments, ~0 first */
@@ -1061,6 +1083,7 @@ int runEbFunction(const char *line)
 				continue;
 			}
 /* Apparently it's the close of an if or an else, just fall through */
+			--nest;
 			goto nextline;
 		}
 
@@ -1119,6 +1142,12 @@ ahead:
 
 /* Here we go! */
 		debugPrint(3, "< %s", new);
+		if (cw->mustrender) {
+			time_t now;
+			time(&now);
+			if (now >= cw->nextrender)
+				rerender(-1);
+		}
 		jClearSync();
 		ok = edbrowseCommand(new, true);
 		free(new);
@@ -1142,6 +1171,8 @@ done:
 		timerspeed = save.timerspeed;
 		dirWrite = save.dw;
 		ls_sort = save.ls_sort;
+		strcpy(lsformat, save.lsformat);
+		showProgress = save.showProgress;
 		down_bg = save.bg;
 		down_jsbg = save.jsbg;
 		iuConvert = save.iu;
@@ -1151,6 +1182,7 @@ done:
 		listNA = save.lna;
 		ls_reverse = save.ls_reverse;
 		searchWrap = save.sw;
+		ebre = save.ebre;
 		allowRedirection = save.hr;
 		sendReferrer = save.sr;
 		allowJS = save.js;
@@ -1164,6 +1196,9 @@ done:
 		caseInsensitive = save.ci;
 		re_utf8 = save.su8;
 		verifyCertificates = save.vs;
+		displayLength = save.ll;
+		formatLineLength = save.fll;
+		formatOverflow = save.fllo;
 	}
 	return rc;
 
@@ -1232,18 +1267,15 @@ void unreadConfigFile(void)
 	nzFree(emojiFile), emojiFile = 0, clearEmojis();
 	nzFree(cookieFile), cookieFile = 0;
 	nzFree(sslCerts), sslCerts = 0;
+	nzFree(pubKey), pubKey = 0;
 	nzFree(downDir), downDir = 0;
 	nzFree(mailDir), mailDir = 0;
 	nzFree(cacheDir);
-	cacheDir = NULL;
+	cacheDir = 0;
 	nzFree(mailUnread), mailUnread = 0;
 	nzFree(mailReply), mailReply = 0;
 
 	webTimeout = mailTimeout = 0;
-	displayLength = 500;
-
-	setDataSource(NULL);
-	setHTTPLanguage(eb_language);
 	delete_ebhosts();
 }
 
@@ -1265,7 +1297,7 @@ static const char *const keywords[] = {
 	"webtimer", "mailtimer", "certfile", "datasource", "proxy",
 	"agentsite", "localizeweb", "imapfetch", "novs", "cachesize",
 	"adbook", "envelope", "emojis", "emoji",
-"include", 0};
+"include", "js", "pubkey", 0};
 
 /* Read the config file and populate the corresponding data structures. */
 /* This routine succeeds, or aborts via one of these macros. */
@@ -1741,9 +1773,11 @@ inside:
 		case 28:	/* nojs */
 			if (*v == '.')
 				++v;
+/* Is this essential?
 			q = strchr(v, '.');
 			if (!q || q[1] == 0)
 				cfgLine1(MSG_EBRC_DomainDot, v);
+*/
 			add_ebhost(v, 'j');
 			continue;
 
@@ -1881,6 +1915,31 @@ inside:
 			f = g;
 			goto top;
 
+		case 45:	/* js */
+			if (*v == '.')
+				++v;
+/* Is this essential?
+			q = strchr(v, '.');
+			if (!q || q[1] == 0)
+				cfgLine1(MSG_EBRC_DomainDot, v);
+*/
+			add_ebhost(v, 'J');
+			continue;
+
+		case 46:	/* pubkey */
+			nzFree(pubKey), pubKey = 0;
+			v = envFileAlloc(v);
+			if(!v) continue;
+			ftype = fileTypeByName(v, false);
+			if (ftype && ftype != 'f')
+				cfgAbort1(MSG_EBRC_KeyNoFile, v);
+			j = open(v, O_RDONLY);
+			if (j < 0)
+				cfgAbort1(MSG_EBRC_KeyNoRead, v);
+			close(j);
+			pubKey = v;
+			continue;
+
 		default:
 			cfgLine1(MSG_EBRC_KeywordNYI, s);
 		}		/* switch */
@@ -1964,7 +2023,7 @@ nokeyword:
 					cfgLine0(MSG_EBRC_NoType);
 				if (!mt->desc)
 					cfgLine0(MSG_EBRC_NDesc);
-				if (!mt->suffix && !mt->prot)
+				if (!mt->suffix && !mt->prot && !mt->content)
 					cfgLine0(MSG_EBRC_NoSuffix);
 				if (!mt->program)
 					cfgLine0(MSG_EBRC_NoProgram);
