@@ -1,6 +1,4 @@
-/* messages.h
- * This file is part of the edbrowse project, released under GPL.
- */
+/* messages.h, symbolic constants for edbrowse messages. */
 
 /*********************************************************************
 Symbolic constants for message numbers.
@@ -95,6 +93,8 @@ this comment block is just the right size.
 *
 *
 *
+*
+*
 *********************************************************************/
 
 enum {
@@ -276,7 +276,7 @@ enum {
 	MSG_YesSlash,
 	MSG_SMBadChar,
 	MSG_BalanceChar,
-	MSG_BalanceNoOpen,
+	MSG_NoAppendURL,
 	MSG_BalanceAmbig,
 	MSG_BalanceNothing,
 	MSG_Unbalanced,
@@ -323,7 +323,7 @@ enum {
 	MSG_BufferXLines,
 	MSG_NoOpen,
 	MSG_NoRead,
-	MSG_LessNull,
+	MSG_WebProtBad,
 	MSG_InputCR,
 	MSG_GopherDownload,
 	MSG_AlreadyInBuffer,
@@ -342,7 +342,7 @@ enum {
 	MSG_IsButton,
 	MSG_SubmitButton,
 	MSG_ResetButton,
-	MSG_Textarea,
+	MSG_BigFile,
 	MSG_Readonly,
 	MSG_InputNewline,
 	MSG_InputLong,
@@ -364,7 +364,7 @@ enum {
 	MSG_InputRange2,
 	MSG_ManyEmptyStrings,
 	MSG_DownAbort,
-	MSG_RexpMissing2,
+	MSG_NoDir0,
 	MSG_SubNumbersMany,
 	MSG_SubSuffixBad,
 	MSG_SubNumberG,
@@ -386,7 +386,7 @@ enum {
 	MSG_BadURL,
 	MSG_DownSuccess,
 	MSG_IdentifyHost,
-	MSG_WebProtBad,
+	MSG_NoPlain,
 	MSG_BufferPreload,
 	MSG_DownProgress,
 	MSG_WebConnect,
@@ -443,7 +443,7 @@ enum {
 	MSG_ABNoAlias2,
 	MSG_RecipNone,
 	MSG_AttSessionEmpty,
-	MSG_AttAccess,
+	MSG_notused346,
 	MSG_AttRegular,
 	MSG_AttEmpty2,
 	MSG_LineDelete1,
@@ -509,7 +509,7 @@ enum {
 	MSG_DBNoTable,
 	MSG_DBBadColumn,
 	MSG_DBManyBlobs,
-	MSG_DBPipes,
+	MSG_DBBackslash,
 	MSG_DBNewline,
 	MSG_DBAddField,
 	MSG_DBLostField,
@@ -762,7 +762,7 @@ enum {
 	MSG_ShowFirst,
 	MSG_Previous,
 	MSG_NoPrevMail,
-	MSG_Textarea0,
+	MSG_Restart,
 	MSG_EBRC_MailDirect,
 	MSG_OptionC,
 	MSG_FileC,
@@ -778,4 +778,19 @@ enum {
 	MSG_EmojiSyntax,
 	MSG_OptStartNone,
 	MSG_OptStartMany,
+	MSG_EndBackslash,
+	MSG_UpdateColname,
+	MSG_FirstColumns,
+	MSG_BreakRow,
+	MSG_DelUnfold,
+	MSG_EbreOff,
+	MSG_EbreOn,
+	MSG_EPlus,
+	MSG_EMinus,
+	MSG_EBRC_KeyNoFile,
+	MSG_EBRC_KeyNoRead,
+	MSG_notused692,
+	MSG_notused693,
+	MSG_DebugTagsOff,
+	MSG_DebugTagsOn,
 };
