@@ -1,15 +1,15 @@
 /* sendmail.c
  * Send mail using the smtp protocol.
  * Send the contents of a file, or the current edbrowse buffer.
- * This file is part of the edbrowse project, released under GPL.
  */
 
 #include "eb.h"
 
 #include <time.h>
 
-#define MAXRECAT 100		/* max number of recipients or attachments */
-#define MAXMSLINE 1024		/* max mail server line */
+#define MAXRECAT 100		// max number of recipients or attachments
+#define MAXMSLINE 1024		// max mail server line
+#define LONGLINELIMIT 76
 
 static char serverLine[MAXMSLINE];
 static bool doSignature;
@@ -269,7 +269,8 @@ static char *charsetString(const char *ct, const char *ce)
  * and the string file is not the filename, but rather, the mail to send. */
 bool
 encodeAttachment(const char *file, int ismail, bool webform,
-		 const char **type_p, const char **enc_p, char **data_p)
+		 const char **type_p, const char **enc_p, char **data_p,
+bool *long_p)
 {
 	char *buf;
 	char c;
@@ -277,8 +278,9 @@ encodeAttachment(const char *file, int ismail, bool webform,
 	char *s, *t, *v;
 	char *ct, *ce;		/* content type, content encoding */
 	int buflen, i, cx;
-	int nacount, nullcount, nlcount;
+	int nacount, nullcount, linelength;
 
+	debugPrint(5, "subject at line %d", ismail);
 	if (ismail < 0) {
 		buf = cloneString(file);
 		buflen = strlen(buf);
@@ -301,7 +303,7 @@ empty:
 				setError(MSG_BufferXEmpty, cx);
 				goto freefail;
 			}
-			sprintf(newfilename, "<buffer %d>", cx);
+			sprintf(newfilename, "<session %d>", cx);
 			file = newfilename;
 			if (sessionList[cx].lw->f0.fileName)
 				file = sessionList[cx].lw->f0.fileName;
@@ -315,11 +317,11 @@ empty:
 				goto freefail;
 			}
 		}
-	}			/* ismail negative or normal */
+	}			// ismail negative or normal
 
 	if (ismail) {
-/* Put newline at the end.  Yes, the buffer is allocated
- * with space for newline and null. */
+// Put newline at the end.  Yes, the buffer is allocated
+// with space for newline and null.
 		if (buf[buflen - 1] != '\n')
 			buf[buflen++] = '\n';
 /* check for subject: line */
@@ -368,7 +370,7 @@ empty:
 				nzFree(subjiso);
 			}
 		}
-		debugPrint(6, "subject = %s", subjectLine);
+		debugPrint(5, "subject = %s", subjectLine);
 /* Blank lines after subject are optional, and ignored. */
 		for (t = buf + buflen; v < t; ++v)
 			if (*v != '\r' && *v != '\n')
@@ -436,7 +438,7 @@ empty:
 // alternative from a buffer is usually html; this doesn't fly if wev
 // send it over as plain text. This is a crude test.
 // Just look for a leading <
-	if(!strncmp(file, "<buffer ", 8)) {
+	if(!strncmp(file, "<session ", 9)) {
 		for (i = 0; i < buflen; ++i) {
 			c = buf[i];
 			if(!isspace(c)) break;
@@ -446,36 +448,30 @@ empty:
 	}
 
 /* Count the nonascii characters */
-	nacount = nullcount = nlcount = 0;
+	nacount = nullcount = linelength = 0;
 	longline = false;
-	s = 0;
-	for (i = 0; i < buflen; ++i) {
-		c = buf[i];
+	for (t = buf, i = 0; i < buflen; ++i, ++t) {
+		c = *t;
 		if (c == '\0')
 			++nullcount;
 		if (c & 0x80)
 			++nacount;
-		if (c != '\n')
+		if (c == '\n') {
+			if(linelength > LONGLINELIMIT) longline = true;
+			linelength = 0;
 			continue;
-		++nlcount;
-		t = buf + i;
-		if (s && t - s > 120)
-			longline = true;
-		if (!s && i > 120)
-			longline = true;
-		s = t;
+		}
+// measure length of line by utf8 characters
+		if(((uchar)c & 0xc0) != 0x80)
+			++linelength;
 	}
-	t = buf + i;
-	if (s && t - s > 120)
-		longline = true;
-	if (!s && i > 120)
-		longline = true;
-	debugPrint(6, "attaching %s length %d nonascii %d nulls %d longline %d",
+	if(linelength > LONGLINELIMIT) longline = true;
+	debugPrint(5, "attaching %s length %d nonascii %d nulls %d longline %d",
 		   file, buflen, nacount, nullcount, longline);
 	nacount += nullcount;
 
 /* Set the type of attachment */
-	if (buflen > 20 && nacount * 5 > buflen) {
+	if (buflen > 20 && nacount * 4 > buflen && !ismail) {
 		if (!ct)
 			ct = "application/octet-stream";	/* default type for binary */
 	}
@@ -486,7 +482,7 @@ empty:
  * files uploaded from a web form need not be encoded, unless they contain
  * nulls, which is a quirk of my slapped together software. */
 
-	if ((!webform && buflen > 20 && nacount * 5 > buflen) ||
+	if ((!webform && buflen > 20 && nacount * 4 > buflen) ||
 	    nullcount) {
 		if (ismail) {
 			setError(MSG_MailBinary, file);
@@ -500,9 +496,9 @@ empty:
 	}
 
 	if (!webform) {
-// Do we need to use quoted-printable?
-// Perhaps this hshould read (nacount > 0)
-		if (nacount * 20 > buflen || nullcount || longline) {
+// Use qp for long lines, it doesn't hurt,
+// and when I send I copy lines into a static buffer of a fixed length.
+		if (longline) {
 			char *newbuf;
 			int l, colno = 0, space = 0;
 
@@ -565,10 +561,12 @@ empty:
 	ce = (nacount ? "8bit" : "7bit");
 
 success:
-	debugPrint(6, "encoded %s %s length %d", ct, ce, strlen(buf));
+	debugPrint(5, "encoded %s %s length %d", ct, ce, strlen(buf));
 	*enc_p = ce;
 	*type_p = ct;
 	*data_p = buf;
+if(long_p) *long_p = longline;
+	debugPrint(6, "%s", buf);
 	return true;
 
 freefail:
@@ -619,19 +617,44 @@ static char *messageTimeID(void)
 	return buf;
 }
 
-static void appendAttachment(const char *s, char **out, int *l)
+static void appendAttachment(const char *s, char **out, int *l, bool longline)
 {
 	const char *t;
 	int n;
+	int paraplus = -1;
 	while (*s) {		/* another line */
 		t = strchr(s, '\n');
 		if (!t)
 			t = s + strlen(s);
 		n = t - s;
-		if (t[-1] == '\r')
+		if (n && t[-1] == '\r')
 			--n;
-		if (n)
+		if(!n) {
+			paraplus = -1;
+		} else {
+			if(longline && paraplus < 0) {
+				paraplus = 0;
+// does this paragraph have any long lines; should we append +
+				const char *u, *v, *x;
+				v = strstr(s, "\n\n"); // end of paragraph
+				x = strstr(s, "\n\r\n");
+				if(!v || (x && x < v)) v = x;
+// a long line will be cut by = by qp encode.
+				u = strstr(s, "=\n");
+				x = strstr(s, "=\r\n");
+				if(!u || (x && x < u)) u = x;
+				if(u && (!v || v > u)) paraplus = 1;
+			}
 			memcpy(serverLine, s, n);
+// if format=flowed, put spaces on the end of lines. Experimental!
+			if(longline && paraplus > 0 && serverLine[n-1] != '=' &&
+			serverLine[n-1] != ' ' &&
+			t[0] && t[1] &&
+			t[1] != '\n' && t[1] != '\r')
+				serverLine[n++] = ' ';
+// We might be able to remove = from a line that ends in space =
+// format=flowed will fix everything up anyways.
+		}
 		serverLine[n] = 0;
 		strcat(serverLine, eol);
 		stringAndString(out, l, serverLine);
@@ -818,7 +841,13 @@ sendMailSMTP(const struct MACCOUNT *account, const char *reply,
 	curl_easy_setopt(handle, CURLOPT_READDATA, &upload);
 	curl_easy_setopt(handle, CURLOPT_UPLOAD, 1L);
 
-	res = curl_easy_perform(handle);
+	if(debugLevel >= 6) {
+		debugPrint(6, "outgoing message");
+		debugPrint(6, "%s", message);
+		puts("debug don't send");
+	} else {
+		res = curl_easy_perform(handle);
+	}
 	if (res == CURLE_OK)
 		smtp_success = true;
 
@@ -840,14 +869,14 @@ sendMail(int account, const char **recipients, const char *body,
 {
 	char *from, *fromiso, *reply;
 	const struct MACCOUNT *a, *ao, *localMail;
-	const char *s, *boundary;
+	const char *s, *s2, *boundary;
 	char reccc[MAXRECAT + MAXCC];
 	char *t;
 	int nat, nrec, cx, i, j;
 	char *out = 0;
 	bool sendmail_success = false;
 	bool mustmime = false;
-	bool firstrec;
+	bool firstrec, longline;
 	const char *ct, *ce;
 	char *encoded = 0;
 
@@ -933,7 +962,7 @@ sendMail(int account, const char **recipients, const char *body,
 		}
 		setError(MSG_ABNoAlias2, s);
 		return false;
-	}			/* recipients */
+	}			// recipients
 
 	if (!j) {
 		setError(MSG_RecipNone);
@@ -952,7 +981,7 @@ sendMail(int account, const char **recipients, const char *body,
 		} else {
 			char ftype = fileTypeByName(s, false);
 			if (!ftype) {
-				setError(MSG_AttAccess, s);
+				setError(MSG_NoAccess, s);
 				return false;
 			}
 			if (ftype != 'f') {
@@ -964,16 +993,16 @@ sendMail(int account, const char **recipients, const char *body,
 				return false;
 			}
 		}
-	}			/* loop over attachments */
+	}			// loop over attachments
 
-	if (!encodeAttachment(body, subjat, false, &ct, &ce, &encoded))
+	if (!encodeAttachment(body, subjat, false, &ct, &ce, &encoded, &longline))
 		return false;
 	if (ce[0] == 'q')
 		mustmime = true;
 
 	boundary = makeBoundary();
 
-/* Build the outgoing mail, as one string. */
+// Build the outgoing mail as one string.
 	out = initString(&j);
 
 	firstrec = true;
@@ -1039,8 +1068,10 @@ sendMail(int account, const char **recipients, const char *body,
 	if (!mustmime) {
 /* no mime components required, we can just send the mail. */
 		sprintf(serverLine,
-			"Content-Type: %s%s%sContent-Transfer-Encoding: %s%s%s",
-			ct, charsetString(ct, ce), eol, ce, eol, eol);
+			"Content-Type: %s%s%s%sContent-Transfer-Encoding: %s%s%s",
+			ct, charsetString(ct, ce),
+			(longline ? "; format=flowed" : ""), eol,
+			ce, eol, eol);
 		stringAndString(&out, &j, serverLine);
 	} else {
 		sprintf(serverLine,
@@ -1053,36 +1084,47 @@ sendMail(int account, const char **recipients, const char *body,
 this format, some or all of this message may not be legible.\r\n\r\n--");
 		stringAndString(&out, &j, boundary);
 		sprintf(serverLine,
-			"%sContent-Type: %s%s%sContent-Transfer-Encoding: %s%s%s",
-			eol, ct, charsetString(ct, ce), eol, ce, eol, eol);
+			"%sContent-Type: %s%s%s%sContent-Transfer-Encoding: %s%s%s",
+			eol, ct, charsetString(ct, ce),
+			(longline ? "; format=flowed" : ""), eol,
+			ce, eol, eol);
 		stringAndString(&out, &j, serverLine);
 	}
 
 /* Now send the body, line by line. */
-	appendAttachment(encoded, &out, &j);
+	appendAttachment(encoded, &out, &j, longline);
 	nzFree(encoded);
 	encoded = 0;
 
 	if (mustmime) {
 		for (i = 0; (s = attachments[i]); ++i) {
-			if (!encodeAttachment(s, 0, false, &ct, &ce, &encoded))
+			if (!encodeAttachment(s, 0, false, &ct, &ce, &encoded, 0))
 				return false;
 			sprintf(serverLine, "%s--%s%sContent-Type: %s%s", eol,
 				boundary, eol, ct, charsetString(ct, ce));
 			stringAndString(&out, &j, serverLine);
-/* If the filename has a quote in it, forget it. */
-/* Also, suppress filename if this is an alternate presentation. */
+// If the filename has a quote in it, forget it.
+// Also, suppress filename if this is an alternate presentation.
+			s2 = 0;
 			if (!nalt && !strchr(s, '"')
 			    && (ismc || stringIsNum(s) < 0)) {
 // for security reasons, and convenience, don't present the absolute path.
-				const char *s2 = strrchr(s, '/');
+				s2 = strrchr(s, '/');
 				if(s2) {
 					if(!*++s2)
 // attaching a directory?  This just shouldn't happen.
 						s2 = 0;
 				} else s2 = s;
+			}
+			if(s2) {
+				sprintf(serverLine, "; name=\"%s\"", s2);
+				stringAndString(&out, &j, serverLine);
+			}
+			if(!nalt) {
+				sprintf(serverLine, "%sContent-Disposition: attachment", eol);
+				stringAndString(&out, &j, serverLine);
 				if(s2) {
-					sprintf(serverLine, "; name=\"%s\"", s2);
+					sprintf(serverLine, "; filename=\"%s\"", s2);
 					stringAndString(&out, &j, serverLine);
 				}
 			}
@@ -1090,7 +1132,7 @@ this format, some or all of this message may not be legible.\r\n\r\n--");
 				"%sContent-Transfer-Encoding: %s%s%s", eol, ce,
 				eol, eol);
 			stringAndString(&out, &j, serverLine);
-			appendAttachment(encoded, &out, &j);
+			appendAttachment(encoded, &out, &j, false);
 			nzFree(encoded);
 			encoded = 0;
 		}		/* loop over attachments */
