@@ -1,6 +1,4 @@
-/* stringfile.c: manage strings, files, and directories for edbrowse.
-* This file is part of the edbrowse project, released under GPL.
- */
+/* stringfile.c: manage strings, files, and directories for edbrowse. */
 
 #include "eb.h"
 
@@ -170,13 +168,13 @@ void spaceCrunch(char *s, bool onespace, bool unprint)
 	if (space && j)
 		--j;		/* drop trailing space */
 	s[j] = 0;
-}				/* spaceCrunch */
+}
 
 /* Like strcpy, but able to cope with overlapping strings. */
 char *strmove(char *dest, const char *src)
 {
 	return (char *)memmove(dest, src, strlen(src) + 1);
-}				/* strmove */
+}
 
 /* OO has a lot of unnecessary overhead, and a few inconveniences,
  * but I really miss it right now.  The following
@@ -210,7 +208,7 @@ void stringAndString(char **s, int *l, const char *t)
 		*s = p;
 	}
 	strcpy(p + oldlen, t);
-}				/* stringAndString */
+}
 
 void stringAndBytes(char **s, int *l, const char *t, int cnt)
 {
@@ -232,7 +230,7 @@ void stringAndBytes(char **s, int *l, const char *t, int cnt)
 	}
 	memcpy(p + oldlen, t, cnt);
 	p[oldlen + cnt] = 0;
-}				/* stringAndBytes */
+}
 
 void stringAndChar(char **s, int *l, char c)
 {
@@ -254,14 +252,14 @@ void stringAndChar(char **s, int *l, char c)
 	}
 	p[oldlen] = c;
 	p[oldlen + 1] = 0;
-}				/* stringAndChar */
+}
 
 void stringAndNum(char **s, int *l, int n)
 {
 	char a[16];
 	sprintf(a, "%d", n);
 	stringAndString(s, l, a);
-}				/* stringAndNum */
+}
 
 /* 64M 16K etc */
 void stringAndKnum(char **s, int *l, int n)
@@ -274,7 +272,7 @@ void stringAndKnum(char **s, int *l, int n)
 	else
 		sprintf(a, "%d", n);
 	stringAndString(s, l, a);
-}				/* stringAndKnum */
+}
 
 char *cloneString(const char *s)
 {
@@ -289,7 +287,7 @@ char *cloneString(const char *s)
 	t = allocString(len);
 	strcpy(t, s);
 	return t;
-}				/* cloneString */
+}
 
 char *cloneMemory(const char *s, int n)
 {
@@ -297,7 +295,7 @@ char *cloneMemory(const char *s, int n)
 	if (n)
 		memcpy(t, s, n);
 	return t;
-}				/* cloneMemory */
+}
 
 void leftClipString(char *s)
 {
@@ -309,7 +307,7 @@ void leftClipString(char *s)
 			break;
 	if (t > s)
 		strmove(s, t);
-}				/* leftClipString */
+}
 
 /* shift string one to the right */
 void shiftRight(char *s, char first)
@@ -318,7 +316,7 @@ void shiftRight(char *s, char first)
 	for (; l >= 0; --l)
 		s[l + 1] = s[l];
 	s[0] = first;
-}				/* shiftRight */
+}
 
 char *Cify(const char *s, int n)
 {
@@ -331,7 +329,7 @@ char *Cify(const char *s, int n)
 			*u = ' ';
 	*u = 0;
 	return t;
-}				/* Cify */
+}
 
 /* pull a substring out of a larger string,
  * and make it its own allocated string */
@@ -344,7 +342,7 @@ char *pullString(const char *s, int l)
 	memcpy(t, s, l);
 	t[l] = 0;
 	return t;
-}				/* pullString */
+}
 
 char *pullString1(const char *s, const char *t)
 {
@@ -484,13 +482,13 @@ int stringInList(const char *const *list, const char *s)
 			++list;
 		}
 	return -1;
-}				/* stringInList */
+}
 
 int stringInListCI(const char *const *list, const char *s)
 {
 	int i = 0;
-	if (!list)
-		i_printfExit(MSG_NullStrListCI);
+	if (!list) return -1;
+//		i_printfExit(MSG_NullStrListCI);
 	if (s)
 		while (*list) {
 			if (stringEqualCI(s, *list))
@@ -499,7 +497,7 @@ int stringInListCI(const char *const *list, const char *s)
 			++list;
 		}
 	return -1;
-}				/* stringInListCI */
+}
 
 int charInList(const char *list, char c)
 {
@@ -510,26 +508,26 @@ int charInList(const char *list, char c)
 	if (!s)
 		return -1;
 	return s - list;
-}				/* charInList */
+}
 
 /* In an empty list, next and prev point back to the list, not to 0. */
 /* We also allow zero. */
 bool listIsEmpty(const struct listHead * l)
 {
 	return l->next == l || l->next == 0;
-}				/* listIsEmpty */
+}
 
 void initList(struct listHead *l)
 {
 	l->prev = l->next = l;
-}				/* initList */
+}
 
 void delFromList(void *x)
 {
 	struct listHead *xh = (struct listHead *)x;
 	((struct listHead *)xh->next)->prev = xh->prev;
 	((struct listHead *)xh->prev)->next = xh->next;
-}				/* delFromList */
+}
 
 void addToListFront(struct listHead *l, void *x)
 {
@@ -538,7 +536,7 @@ void addToListFront(struct listHead *l, void *x)
 	xh->prev = l;
 	l->next = (struct listHead *)x;
 	((struct listHead *)xh->next)->prev = (struct listHead *)x;
-}				/* addToListFront */
+}
 
 void addToListBack(struct listHead *l, void *x)
 {
@@ -547,7 +545,7 @@ void addToListBack(struct listHead *l, void *x)
 	xh->next = l;
 	l->prev = (struct listHead *)x;
 	((struct listHead *)xh->prev)->next = (struct listHead *)x;
-}				/* addToListBack */
+}
 
 void addAtPosition(void *p, void *x)
 {
@@ -673,18 +671,18 @@ int perl2c(char *t)
 }				/* perl2c */
 
 /* The length of a perl string includes its terminating newline */
-unsigned pstLength(pst s)
+unsigned pstLength(const pst s)
 {
-	pst t;
+	const uchar *t;
 	if (!s)
 		i_printfExit(MSG_NullPtr);
 	t = s;
 	while (*t != '\n')
 		++t;
 	return t + 1 - s;
-}				/* pstLength */
+}
 
-pst clonePstring(pst s)
+pst clonePstring(const pst s)
 {
 	pst t;
 	unsigned len;
@@ -694,7 +692,7 @@ pst clonePstring(pst s)
 	t = (pst) allocMem(len);
 	memcpy(t, s, len);
 	return t;
-}				/* clonePstring */
+}
 
 // Strings are assumed distinct, hence the use of memcpy.
 void copyPstring(pst s, const pst t)
@@ -721,6 +719,14 @@ bool fdIntoMemory(int fd, char **data, int *len)
 
 	n = 0;
 	do {
+		if(length >= 0x7fffff00 - blocksize) {
+			nzFree(buf);
+			nzFree(chunk);
+			*data = emptyString;
+			*len = 0;
+			setError(MSG_BigFile);
+			return false;
+		}
 		n = read(fd, chunk, blocksize);
 		if (n < 0) {
 			nzFree(buf);
